#ifndef ADCCALIBRATIONDIALOG_H
#define ADCCALIBRATIONDIALOG_H

#include <QDialog>
#include <QMap>

#include "base/DeviceIndex.h"
#include "fsm/FsmState.h"
#include "lib-common/BaseDeviceAppConfig.h"

class QTableWidget;
struct CalResult;
struct ConfigKey;
struct DeviceIndexEnabledMap;

namespace Ui {
class AdcCalibrationDialog;
}

class AdcCalibrationDialog : public QDialog
{
    Q_OBJECT

public:
    explicit AdcCalibrationDialog(QWidget *parent = nullptr);
    ~AdcCalibrationDialog() override;

signals:
    void performCalibrate() const;
    void configChanged(const BaseDeviceAppConfig &) const;

public slots:
    void onFsmStateChange(const FsmState &state);
    void setCalResult(const CalResult &newCalResult);
    void setConfig(const ConfigKey &key, const BaseDeviceAppConfig &newConfig);

private slots:
    void on_pushButtonClear_clicked();
    void on_pushButtonStartStop_clicked();

private:
    int getTabIndex(const DeviceIndex &i) const;
    QTableWidget *getTableWidget(const DeviceIndex &i) const;
    void addTab(int tabIndex, const DeviceIndex &i) const;
    void updateTable(const DeviceIndex &index) const;

    Ui::AdcCalibrationDialog * const ui;
    DeviceIndexSet devSet;
    BaseDeviceAppConfig config;
    FsmState fsmState = FsmState::Unknown;
};

#endif // ADCCALIBRATIONDIALOG_H
