#include "DeviceSettingsDialog.h"
#include "ModularDeviceConfig.h"
#include "ui_DeviceSettingsDialog.h"

#include <QJsonDocument>
#include <QJsonObject>

static QString DominoSettingsToString(const DominoSettings &config)
{
    const auto &qvar = QVariant::fromValue(config);
    const QJsonValue &json = qvar.value<QJsonValue>(); // conversion
    return QJsonDocument(json.toObject()).toJson(QJsonDocument::Indented);
}

DeviceSettingsDialog::DeviceSettingsDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::DeviceSettingsDialog)
{
    ui->setupUi(this);
    setWindowTitle("Device settings");
}

DeviceSettingsDialog::~DeviceSettingsDialog()
{
    delete ui;
}

void DeviceSettingsDialog::deviceListUpdated(const QMap<DeviceIndex, ModularDeviceConfig> &devMap)
{
    devices = devMap;
    QListWidget *list = ui->listWidgetDevices;
    while (list->count()) {
        QListWidgetItem *item = list->takeItem(0);
        delete item;
    }

    QSet<int> mtuSet;
    for(auto it=devices.cbegin(); it!=devices.cend(); ++it) {
        if (!it->enabled) continue;
        new QListWidgetItem(getDevName(it.key()), list);
        mtuSet << it->networkPort.mtu;
    }

    QStringList mtuList;
    for(auto v : mtuSet)
        mtuList << QString::number(v);

    ui->labelMtuSize->setText(mtuList.join(", "));
}

void DeviceSettingsDialog::setStopConditions(int limit)
{
    ui->labelStopConditionsVal->setText(limit ? QString("EvNum limit(%1)").arg(limit)
                                           : "disabled");
    ui->labelStopConditions->setEnabled(limit);
    ui->labelStopConditionsVal->setEnabled(limit);
}

void DeviceSettingsDialog::on_listWidgetDevices_currentItemChanged(QListWidgetItem *current, QListWidgetItem *)
{
    QPlainTextEdit *devInfo = ui->plainTextEditInfo;
    devInfo->clear();

    if(!current)
        return;

    for(const DeviceIndex& index : devices.keys()) {
        const auto &ds = devices[index];
        if (!ds.enabled) continue;
        if(current->text() == getDevName(index)){
            devInfo->setPlainText(DominoSettingsToString(ds.adcSystemCore));
            break;
        }
    }
}

QString DeviceSettingsDialog::getDevName(const DeviceIndex &index) const
{
    return QString("%1 %2")
        .arg(DeviceIndex::getDeviceIdName(index.getDevId()))
        .arg(index.getSerialStr());
}
