#include "DreConfigConverter.h"
#include "DreConfig.h"

#include "base/ConfigConverterUtil.h"

constexpr const char *k_sdbId = "sdbId";
constexpr const char *k_sdbSequence = "sdbSeq";
constexpr const char *k_regOffset = "regOffset";
constexpr const char *k_isWrite = "isWrite";
constexpr const char *k_regType = "regType";
constexpr const char *k_writeVal = "writeVal";
constexpr const char *k_comment = "comment";


constexpr const char *k_deviceIndex = "deviceIndex";
constexpr const char *k_isCustom = "isCustom";
constexpr const char *k_regAddr = "regAddr";
constexpr const char *k_regWriteVal = "regWriteVal";
constexpr const char *k_regOpActions = "regOpActions";

// ===================== RegOpAction Start =====================

template<>
bool AbstractConfigConverter<RegOpAction>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<RegOpAction>::toJsonObject(const RegOpAction &in)
{
    QVariantMap m;
    m[k_sdbId] = QVariant::fromValue(in.sdbId);
    m[k_sdbSequence] = QVariant::fromValue(in.sdbSeq);
    m[k_regOffset] = QVariant::fromValue(in.regOffset);
    m[k_regType] = QVariant::fromValue(static_cast<int>(in.regType));
    m[k_isWrite] = QVariant::fromValue(in.isWrite);
//    m[k_writeVal] = QVariant::fromValue(in.writeVal);
    m[k_writeVal] = QVariant::fromValue(in.writeVal);
    m[k_comment] = QVariant::fromValue(in.comment);
    return fromVariantMap(m);
}

template<>
RegOpAction AbstractConfigConverter<RegOpAction>::fromJsonObject(const QJsonObject &m)
{
    RegOpAction out;
    update_value(m, k_sdbId, &out.sdbId);
    update_value(m, k_sdbSequence, &out.sdbSeq);
    update_value(m, k_regOffset, &out.regOffset);
    int regType;
    update_value(m, k_regType, &regType);
    out.regType = static_cast<REG_OP_ACTION_TYPE>(regType);
    update_value(m, k_isWrite, &out.isWrite);
    update_value(m, k_writeVal, &out.writeVal);
    update_value(m, k_comment, &out.comment);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<RegOpAction>::toJsonValue(const RegOpAction &in)
{
    return toJsonObject(in);
}

template<>
RegOpAction AbstractConfigConverter<RegOpAction>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<RegOpAction>::registerSubTypes()
{
}

// ===================== RegOpAction End =====================
// ===================== DreConfig Start =====================

template<>
bool AbstractConfigConverter<DreConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<DreConfig>::toJsonObject(const DreConfig &in)
{
    QVariantMap m;
    m[k_deviceIndex] = QVariant::fromValue(in.deviceIndex);
    m[k_isCustom] = QVariant::fromValue(in.isCustom);
    m[k_regType] = QVariant::fromValue(static_cast<int>(in.regType));
    m[k_regAddr] = QVariant::fromValue(in.regAddr);
    m[k_regWriteVal] = jsonValueFromUInt64(in.regWriteVal);
    m[k_regOpActions] = ConfigConverterUtil::toJsonValue(in.regOpActions);
    return fromVariantMap(m);
}

template<>
DreConfig AbstractConfigConverter<DreConfig>::fromJsonObject(const QJsonObject &m)
{
    DreConfig out;
    update_value(m, k_deviceIndex, &out.deviceIndex);
    update_value(m, k_isCustom, &out.isCustom);
    int v;
    update_value(m, k_regType, &v);
    out.regType = static_cast<ConfigBase::REG_TYPE>(v);
    update_value(m, k_regAddr, &out.regAddr);
    out.regWriteVal = jsonValueToUInt64(m[k_regWriteVal]);
    out.regOpActions = ConfigConverterUtil::toList<RegOpAction>(m[k_regOpActions]);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<DreConfig>::toJsonValue(const DreConfig &in)
{
    return toJsonObject(in);
}

template<>
DreConfig AbstractConfigConverter<DreConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<DreConfig>::registerSubTypes()
{
    AbstractConfigConverter<DeviceIndex>::registerTypes();
    AbstractConfigConverter<RegOpAction>::registerTypes();
}

// ===================== DreConfig End =====================
