#include "RegOpTableView.h"

#include <QComboBox>
#include <QDebug>
#include <QHeaderView>

#include "HexSpinBoxDelegate.h"
#include "RegOpColumns.h"
#include "SdbComboBoxDelegate.h"
#include "RegTypeComboBoxDelegate.h"

namespace {
static const int SDB_COL = static_cast<int>(RegOpColumns::COL_SDB_ID);
static const int REG_TYPE_COL = static_cast<int>(RegOpColumns::COL_REG_TYPE);
static const int REG_OFFSET_COL = static_cast<int>(RegOpColumns::COL_REG_OFFSET);
}

RegOpTableView::RegOpTableView(QWidget *parent)
    : QTableView(parent)
    , hexSbDelegate( new HexSpinBoxDelegate(this))
    , regTypeDelegate(new RegTypeComboBoxDelegate(this))
    , sdbDelegate(new SdbComboBoxDelegate(this))
{
#if QT_VERSION >= 0x050000
    horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
#else
    horizontalHeader()->setResizeMode(QHeaderView::ResizeToContents);
#endif
}

void RegOpTableView::setModel(QAbstractItemModel *model)
{
    QTableView::setModel(model);

    setItemDelegateForColumn(REG_OFFSET_COL, hexSbDelegate);
    setItemDelegateForColumn(REG_TYPE_COL, regTypeDelegate);
    setItemDelegateForColumn(SDB_COL, sdbDelegate);

    connect(model, &QAbstractItemModel::modelReset,
            this, &RegOpTableView::addDelegate);
    connect(model, &QAbstractItemModel::rowsInserted,
            this, &RegOpTableView::addDelegate);
}

void RegOpTableView::setSdbMap(const QMap<QString, QPair<quint32, quint32> > &_sdbMap)
{
    if(sdbMap == _sdbMap)
        return;
    sdbMap = _sdbMap;
    sdbDelegate->setSdbMap(sdbMap);

    const auto m = model();
    for(int r=0; r<model()->rowCount(); ++r) {
        const auto &i = m->index(r, SDB_COL);
        if(!isPersistentEditorOpen(i))
            continue;
        auto w = indexWidget(i);
        if(!w)
            continue;
        auto cb = qobject_cast<QComboBox*>(w);
        if(cb) {
            auto val = m->data(i, Qt::EditRole).toList();
            SdbComboBoxDelegate::updateCbItems(cb, sdbMap, val);
        }
    }
    resizeColumnToContents(SDB_COL);
}

void RegOpTableView::addDelegate()
{
    const auto m = model();
    for(auto c : {SDB_COL, REG_TYPE_COL, REG_OFFSET_COL}) {
        for(int r=0; r<model()->rowCount(); ++r) {
            const auto &i = m->index(r, c);
            if(!isPersistentEditorOpen(i))
                openPersistentEditor(i);
        }
        resizeColumnToContents(c);
    }
//    clearSelection();
}
