#ifndef HRBANMAINWINDOW_H
#define HRBANMAINWINDOW_H

#include <QMainWindow>
#include <QMap>
#include <QTimer>

#include "pnp-server/PNPMessage.h"

class QHostAddress;
class QLabel;
class QThread;

class QwtPlot;

class FileReader;
class HrbAnalyzer;
class HrbPlot;
class TcpClient;
class QSemaphore;

namespace Ui {
class HrbAnMainWindow;
}


class HrbAnMainWindow : public QMainWindow
{
    Q_OBJECT
public:
    explicit HrbAnMainWindow(QWidget *parent = 0);
    ~HrbAnMainWindow();
    void init(QStringList args);

public slots:
    void pnpCacheUpdated(const PnpInfoCache &cache);

private Q_SLOTS:
    void openFile();
    void hrbCountersUpdated(quint32 serial, QVector<QVector<quint64> > data, quint32 trigPassed);
    void on_pushButtonClear_clicked();
    void on_checkBoxAutoClear_toggled( bool state );
    void on_comboBoxShowTrg_currentTextChanged(const QString &arg1);
    void on_comboBoxHistGroupping_currentTextChanged(const QString &);
    void on_spinBoxHistLimit_valueChanged(int arg1);
    void on_pushButtonAdd_clicked();
    void on_actionConnect_to_data_source_triggered();
    void on_actionReorganize_plots_triggered();
    void on_actionRelease_missing_plots_triggered();
    void on_actionChannels_toggled(bool isByChannels);
    void endOfSpill(int spillNumber);
    void eventNumUpdated(quint32 eventNumber);

signals:
    void clear(quint32 devId=0) const;
    void closing() const;
    void readerFileNameUpdated(const QString &) const;
    void groppingUpdated(int type) const;
    void tcpAddressChanged(const QHostAddress &, uint) const;
    void autoClearUpdated(bool) const;

private:
    void addPlot(quint32 serial);
    void updateClearTargets();
    void closeEvent(QCloseEvent *event);
    void setStatus(const QString &status);
    void updateHistogram();
    void testAdd(quint64 id);
    void saveConfig();
    bool hasFreeProgs() const;
    void updateTcpInterface(const ProgramDescription &descr);
    void updateSourceLabel();

    Ui::HrbAnMainWindow *ui;
    QSemaphore *const queueFreeSizeSem;
    HrbAnalyzer * const hrbAnalyzer;
    QThread *const hrbAnalyzerThread;
    FileReader *reader;
    QString outputFileName;
    QTimer autoClearTimer;
    TcpClient * const tcpClient;
    QThread *const tcpClientThread;
    QMap<quint32, HrbPlot *> plots;
    QMap<quint32, QPair<int,int> > places; // <row, column>
    PnpInfoCache pnpCache;
    ProgramDescription selectedProgram;
    ProgramInterface selectedInterface;
    QLabel * const sourceLable;
    QLabel * const eventNumLable;
    QLabel * const spillNumLable;
    quint32 curDeviceId;
    QMap<quint32, int> histLimit;
};

#endif // HRBANMAINWINDOW_H
