#ifndef HRBANALYZER_H
#define HRBANALYZER_H

#include <QByteArray>
#include <QObject>
#include <QMap>
#include <QVector>
#include <QTime>
#include <QTimer>
#include <QMetaType>

class QSemaphore;
class QDateTime;

struct HrbData
{
    HrbData();
    void clear() { curCnt.clear(); tmpCnt.clear(); modified=true; sumTrigPassed=0; }

    QVector< QVector<quint64> > curCnt;
    QVector< QVector<quint64> > tmpCnt;
    quint32 sumTrigPassed;
    QTime lastSendTime;
    bool modified;
};
Q_DECLARE_METATYPE(QVector< QVector<quint64> >)

#include "DataTypes.h"

#define MStreamVersion 2

class HrbAnalyzer : public QObject
{
    Q_OBJECT
public:
    HrbAnalyzer(QSemaphore *sem, QObject *p = 0);

public slots:
    void dataAcived(QByteArray rawData);
    void clear(quint32 devId = 0);
    void autoClearUpdated(bool autoClr);

private slots:
    void setAutoClear(bool state) { autoClear = state; }
    void sendResults(bool force=false);
//    void updateEnabled();
    void checkParent();

signals:
    void countersUpdated(quint32 curDevId, QVector< QVector<quint64> > cur, quint32 trig);
    void endOfSpill(int spillNum) const;
    void eventNumUpdated(quint32 globEvNum) const;

private:
    bool skipData();
    double testDummy();
    void read();
    void flushCounters();
    bool findNextEventHeader();
    quint32 readDevice();
    quint32 readEvent(); // returns handaled size

    int index;
    QSemaphore *queueFreeSizeSem;
    QByteArray rawData;

    MpdHdr *mpdHdr;
    MpdEventHdr *mpdEventHdr;
    MpdDeviceHdr *mpdDeviceHdr;
    MpdMStreamHdr *mpdMStreamHdr;
    HrbHeader *hrbHeader;
    HrbSample *sample;

    quint32 lastEventNumAtPrevFrame;
    quint32 prevEventNum;
    quint32 sumTrigs; // Number of trigger for previous spill
    uint curFrameStartPos;
    quint64 absPos;
    int curPos;
    int handledSize;
    QMap<quint32, HrbData > hrbData;
    QTimer sendEndingTimer;
    QTimer checkParentTimer;
    bool autoClear;
    bool readyForAutoClear;
    bool firstFrame;
    bool previousProceed;
    bool enabled;
    int lastSampleNumber;
    int spillNumber;
};

#endif // HRBANALYZER_H
