#include "ReorganizationDialog.h"
#include "ui_ReorganizationDialog.h"

#include <QComboBox>
#include <QDebug>

ReorganizationDialog::ReorganizationDialog(QMap<quint32, QPair<int,int> > p, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ReorganizationDialog),
    places(p),
    rowCount(0),
    columnCount(0)
{
    ui->setupUi(this);
    int maxRow=0, maxColumn=0;
    foreach (quint32 serial, places.keys()) {
        elements[serial] = QString("0x%1").arg(serial, 8, 16, QChar('0'));
        if(maxRow < places[serial].first)
            maxRow = places[serial].first;
        if(maxColumn < places[serial].second)
            maxColumn= places[serial].second;
//        qDebug()<<serial<<"="<<places[serial].first<<":"<<places[serial].second;
    }
    elements[0] = "empty";

    for(int r=0; r <= maxRow; ++r) {
        for(int c=0; c <= maxColumn; ++c) {
            QComboBox * box = new QComboBox(this);
            foreach (quint32 serial, elements.keys()) {
                box->insertItem(serial, elements[serial], serial);
            }

            box->setCurrentIndex(0);
            foreach (quint32 ser, places.keys()) {
                if(places[ser].first == r && places[ser].second == c) {
                    box->setCurrentText(elements[ser]);
//                    qDebug()<<"[Set init index]\t"<<r<<":"<<c<<"="<<hex<<ser;
                    break;
                }
            }
            connect(box, SIGNAL(currentIndexChanged(int)), SLOT(boxCurrentIndexChanged(int)));
            ui->gridLayout->addWidget(box, r, c);
            rowCount = qMax(rowCount, r+1);
            columnCount = qMax(columnCount, c+1);
        }
    }

    ui->spinBoxRows->setValue(maxRow+1);
    ui->spinBoxColumns->setValue(maxColumn+1);
    ui->spinBoxRows->setMaximum(20);
    ui->spinBoxColumns->setMaximum(20);
}

ReorganizationDialog::~ReorganizationDialog()
{
    delete ui;
}

void ReorganizationDialog::boxCurrentIndexChanged(int index)
{
    bool found=false;
    QComboBox *curBox = qobject_cast<QComboBox *>(sender());
    int row=0, col=0; // current Box position;
    for(row=0; row < rowCount; ++row){
        for(col=0; col < columnCount; ++col){
            if(ui->gridLayout->itemAtPosition(row, col)->widget() == sender()){
                found = true;
                break;
            }
        }
        if(found) break;
    }
    Q_ASSERT(found);

    quint32 newSer = curBox->currentData().toUInt();
    quint32 prevSer = 0;
    foreach (quint32 ser, places.keys()) {
        if(places[ser].first == row && places[ser].second == col){
            prevSer = ser;
            break;
        }
    }
//    qWarning() << "Box "<<row<<":"<<col<<" change "<<prevSer<<" to "<< newSer;
    if(newSer==0) {
        int i = curBox->findData(prevSer);
        curBox->blockSignals(true);
        curBox->setCurrentIndex(i);
        curBox->blockSignals(false);
    } else {
        QComboBox *prevBox=NULL;
        prevBox = qobject_cast<QComboBox *>(ui->gridLayout->itemAtPosition(places[newSer].first, places[newSer].second)->widget());
        Q_ASSERT(prevBox);
//        foreach (QComboBox *box, this->findChildren<QComboBox *>()) {
//            if(!box || box == sender()) continue;
//            quint32 data = box->currentData().toUInt();
//            if(data == newSer){
//                prevBox = box;
//                break;
//            }
//        }
//        Q_ASSERT(prevBox);
        int i = prevBox->findData(prevSer);
        prevBox->blockSignals(true);
        prevBox->setCurrentIndex(i);
        prevBox->blockSignals(false);
        if(prevSer){
            places[prevSer] = places[newSer];
        }
        places[newSer].first = row;
        places[newSer].second = col;
    }
}

void ReorganizationDialog::on_spinBoxRows_valueChanged(int rows)
{
    if(rowCount == rows) return;
    if(rowCount < rows) {
        for(int row=rowCount; row < rows; ++row){
            for(int col=0; col < columnCount; ++col){
                QComboBox * box = new QComboBox(this);
                foreach (quint32 serial, elements.keys()) {
                    box->insertItem(serial, elements[serial], serial);
                }

                box->setCurrentIndex(0);
                connect(box, SIGNAL(currentIndexChanged(int)), SLOT(boxCurrentIndexChanged(int)));
                ui->gridLayout->addWidget(box, row, col);
                rowCount = qMax(rowCount, row+1);
                columnCount = qMax(columnCount, col+1);
            }
        }
    } else {
        int maxRow = 0;
        foreach (quint32 ser, places.keys())
            maxRow = qMax(maxRow, places[ser].first);
        if(rows < maxRow+1){
            rows = maxRow+1;
            ui->spinBoxRows->blockSignals(true);
            ui->spinBoxRows->setValue(rows);
            ui->spinBoxRows->blockSignals(false);
        }

        for(int row = rowCount-1; row >= rows; --row){
            for(int col = 0; col < columnCount; ++col){
                QLayoutItem *item = ui->gridLayout->itemAtPosition(row,col);
                if(!item) continue;
                ui->gridLayout->removeItem(item);
                delete item->widget();
                delete item;
            }
        }
        rowCount = rows;
    }
}

void ReorganizationDialog::on_spinBoxColumns_valueChanged(int columns)
{
    if(columnCount == columns) return;
    if(columnCount < columns) {
        for(int col=columnCount; col < columns; ++col){
            for(int row=0; row < rowCount; ++row){
                QComboBox * box = new QComboBox(this);
                foreach (quint32 serial, elements.keys()) {
                    box->insertItem(serial, elements[serial], serial);
                }

                box->setCurrentIndex(0);
                connect(box, SIGNAL(currentIndexChanged(int)), SLOT(boxCurrentIndexChanged(int)));
                ui->gridLayout->addWidget(box, row, col);
                rowCount = qMax(rowCount, row+1);
                columnCount = qMax(columnCount, col+1);
            }
        }
    } else {
        int maxCol = 0;
        foreach (quint32 ser, places.keys())
            maxCol = qMax(maxCol, places[ser].second);
        if(columns < maxCol+1){
            columns = maxCol+1;
            ui->spinBoxColumns->blockSignals(true);
            ui->spinBoxColumns->setValue(columns);
            ui->spinBoxColumns->blockSignals(false);
        }

        for(int col = columnCount-1; col >= columns; --col){
            for(int row = 0; row < rowCount; ++row){
                QLayoutItem *item = ui->gridLayout->itemAtPosition(row,col);
                if(!item) continue;
                ui->gridLayout->removeItem(item);
                delete item->widget();
                delete item;
            }
        }
        columnCount = columns;
    }
}
