//
//    Copyright 2013 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef HRBMAINWINDOW_H
#define HRBMAINWINDOW_H

#include "app/BaseMainWindow.h"
#include "base/RcRunId.h"
#include "config/HrbAppConfig.h"
#include "fsm/FsmState.h"

class QLabel;
class QtMregDevice;
class QElapsedTimer;
class QTimer;
class ProcessMonitor;
class QTableWidget;
class QTcpSocket;

class ConfigKey;
class ConfigSelector;
class DeviceIndex;
class DiscoverDialog;
class GenericHardware;
class HrbDevice;
class HrbStatus;
class RemoteControlServer;
struct RemoteControlServerStatus;
class StatisticDialog;
class TtbDevice;
class TtbStatus;
//class DevFwRevMap;
class ConfigurationManager;
class ConfigIndexName;
class MStreamStat;
class MStreamStatisticDialog;
class HrbModel;
class TtbModel;
class TriggerSetupDialog;

namespace Ui {
class HrbMainWindow;
}

class HrbMainWindow : public BaseMainWindow
{
    Q_OBJECT
public:
    explicit HrbMainWindow(QWidget *parent = nullptr);
    ~HrbMainWindow();

public slots:
    void setConfig(const ConfigKey &config_key, const HrbAppConfig &newConfig);
    void onRemoteStatusUpdated(const RemoteControlServerStatus &st);
    void preparingRun(const RcRunId &_runId);
    void onFsmStateChange(const FsmState &state);
    void configSelectorChanged(const ConfigIndexName &cin);

private slots:
    // Gui actions
    void on_pushButtonStart_clicked();
    void on_pushButtonStop_clicked();

    void on_actionTriggerSetup_triggered();
    void on_actionBoardsDetails_triggered();
    void on_actionMStream_statistic_triggered();
    void on_actionPrint_FSM_inputs_triggered();
    void on_actionFindDevices_triggered();
    void on_spinBoxThreshold_valueChanged(double val);
    void on_spinBoxLatency_valueChanged(int val);
    void on_spinBoxWindow_valueChanged(int val);
    void on_spinBoxMinHitDuration_valueChanged(int val);
    void on_actionAllow_Remote_Control_toggled(bool checked);
    void on_actionShow_average_busy_time_toggled(bool showAveraged);

    void on_actionReset_MStream_triggered() const;
    void on_actionSend_soft_reset_triggered() const;
    void on_actionConfiguration_manager_triggered() const;


signals:
    void runError(QString,int,QString) const;
    void runWarning(QString runIndex, int runNumber, QString warnStr);
    void deleteAll() const;
    void sendRemoteState(QString cmd, bool read4Run) const;
    void completedRemoteStart(bool success, QString runIndex, int runNumber);
    void completedRemoteStop(bool success, QString runIndex, int runNumber);
    void updateHrbSetup(const ModularDeviceConfig &, const HrbTriggerSetup &, uint setupSeqIndex) const;
    void setStatusIndex(uint setupSeqIndex) const;
    void updateTtbSetup(const ModularDeviceConfig &, uint setupSeqIndex) const;
//    void saveRunToDB(QString runIndex, uint runNumber, const HrbAppConfig &s, const DevFwRevMap &devFwRev);
    void configChanged(const HrbAppConfig &config) const;
    void printFsmInputs() const;
    void switch_to_config(const ConfigIndexName &conf) const;
    void softReset() const;
    void mstream_reset_all(int seqNum) const;
    void performLocalStart() const;
    void performLocalStop() const;

protected:
    virtual void saveAppGuiSettings(QSettings &s) const override;
    virtual void loadAppGuiSettings(const QSettings &s) override;

private:
    void updateGuiVal();
    void updateLabelRemoteState() const;
    void updateControlState();

    void initTtbTable();

    bool resetAll();
    QStringList hostStrList() const;
    void sendConfig() const;

    bool isHrbStatusOk(const HrbStatus &status, bool silent = false) const;
    TriggerSetupDialog *getTriggerSetupDialog() const;

    Ui::HrbMainWindow *ui;
    ConfigurationManager * const configuration_manager;
    DiscoverDialog * const discoverDialog;
    StatisticDialog * const statisticDialog;
    MStreamStatisticDialog * const mStreamStatisticDialog;

    QLabel * const labelFsmState;
    HrbModel * const hrbModel;
    TtbModel * const ttbModel;

    HrbAppConfig config;
    int blockControl = 0;
    bool underRemoteCtrl = false;
    RcRunId runId;
    FsmState fsmState = FsmState::Init;
    bool status_request_flag = false;
};

#endif // HRBMAINWINDOW_H
