#include "HrbTableView.h"

#include <QHeaderView>

#include "HrbModelColumns.h"
#include "HrbSpinBoxDelegate.h"

HrbTableView::HrbTableView(QWidget *parent)
    :QTableView(parent)
    ,delegate(new HrbSpinBoxDelegate(this))
{
#if QT_VERSION >= 0x050000
    horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
#else
    horizontalHeader()->setResizeMode(QHeaderView::ResizeToContents);
#endif
}

void HrbTableView::setModel(QAbstractItemModel *model)
{
    QTableView::setModel(model);
    hideColumn(COL_HRB_WRITE);
    hideColumn(COL_HRB_IP);
    hideColumn(COL_HRB_TIME);
    for(int col=COL_HRB_ASD_FIRST; col<=COL_HRB_ASD_LAST; ++col)
        setItemDelegateForColumn(col, delegate);

    connect(model, &QAbstractItemModel::modelReset, this, &HrbTableView::addDelegate);
    connect(model, &QAbstractItemModel::rowsInserted, this, &HrbTableView::addDelegate);
}

void HrbTableView::addDelegate()
{
    const auto m = model();
    for(int r=0; r<model()->rowCount(); ++r) {
        for(int c=COL_HRB_ASD_FIRST; c<=COL_HRB_ASD_LAST; ++c) {
            const auto &i = m->index(r, c);
            if(!isPersistentEditorOpen(i))
                openPersistentEditor(i);
        }
    }
    clearSelection();
}
