#ifndef STATISTICDIALOG_H
#define STATISTICDIALOG_H

#include <QDialog>

class QSettings;
//class QTableView;
class QTextEdit;

class ConfigIndexName;
struct DeviceIndexEnabledMap;

#include "hardware/HrbStatus.h"
#include "hardware/TtbStatus.h"

class StatisticDialog : public QDialog
{
    Q_OBJECT
public:
    explicit StatisticDialog(QWidget *parent = 0);
    static QDateTime getDateTimeFromClock(const quint64 ts, quint64 *us=NULL);
    void clearStatistic();
    void saveSettings(QSettings &s) const;
    void loadSettings(const QSettings &s);

signals:
    void switch_to_config(const ConfigIndexName &conf) const;
    void deviceListChanged(const DeviceIndexEnabledMap &);

public slots:
    void setDeviceList(const DeviceIndexEnabledMap &newList);
    void hrbStatusUpdated(const HrbStatus&);
    void ttbStatusUpdated(const TtbStatus&);
    virtual void setVisible(bool visible) override;
    void configSelectorChanged(const ConfigIndexName &conf);

private slots:
    virtual void reject() override;
    void refreshTable();

protected:
    void resizeEvent(QResizeEvent *e) override;

private:
    void addHrbInfo();
    void addTtbInfo();

    QTextEdit *editor;
//    QTableView *view;
    QStringList html;
    DeviceIndexSet devSet;
    QMap<DeviceIndex, HrbStatus> hrbStatusCurrent;
    QMap<DeviceIndex, HrbStatus> hrbStatusLast;
    QMap<DeviceIndex, HrbStatus> hrbStatusDif;
    QMap<DeviceIndex, TtbStatus> ttbStatusCurrent;
    QMap<DeviceIndex, TtbStatus> ttbStatusLast;
    QMap<DeviceIndex, TtbStatus> ttbStatusDif;
    QTimer * const refreshTimer;
};

#endif // STATISTICDIALOG_H
