//
//    Copyright 2013 AFI Electronics
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "HrbAppConfig.h"

#include <QDebug>
#include "base/DeviceIndexEnabled.h"

bool HrbAppConfig::operator ==(const HrbAppConfig &setup) const
{
    return
            defaultHrb == setup.defaultHrb &&
            hrb == setup.hrb &&
            ttb == setup.ttb &&
            triggerSetup == setup.triggerSetup &&
            allowRemoteControl == setup.allowRemoteControl &&
            showAveragedBusy == setup.showAveragedBusy &&
            true;
}

double HrbAppConfig::getThreshold() const
{
    double ret = defaultHrb.hrb.thr.first();
    for(auto c : hrb) {
        ret = c.hrb.thr.first();
        if(c.enabled)
            return ret;
    }
    return ret;
}

void HrbAppConfig::setRunState(bool run)
{
    for(auto &c : hrb)
        c.run = run && c.enabled;
    for(auto &c : ttb)
        c.run = run && c.enabled;
}

void HrbAppConfig::setThreshold(double val)
{
    defaultHrb.hrb.setThreshold(val);
    for(auto &cfg : hrb)
        if(cfg.enabled)
            cfg.hrb.setThreshold(val);
}

void HrbAppConfig::spreadGlobalCfg()
{
    for(auto &cfg : hrb)
        cfg.hrb.spreadGlobalCfg(defaultHrb.hrb);
}

DeviceIndexEnabledMap HrbAppConfig::getDeviceIndexEnabledMap(DeviceFilter filter) const
{
    DeviceIndexEnabledMap dieMap;

    if(filter == DeviceFilter::All || filter == DeviceFilter::HRB) {
        for(auto &c : hrb)
            dieMap[c.index] = {c.index, c.enabled};
    }

    if(filter == DeviceFilter::All || filter == DeviceFilter::TTB) {
        for(auto &c : ttb)
            dieMap[c.index] = {c.index, c.enabled};
    }
    return dieMap;
}

bool HrbAppConfig::isRun() const
{
    int on=0, off=0;
    for(auto &ds : hrb) {
        if(ds.enabled)
            ++(ds.isRun() ? on : off);
    }
    for(auto &ds : ttb) {
        if(ds.enabled)
            ++(ds.isRun() ? on : off);
    }
    if(on && off)
        qWarning() << "Config has mix (running/stopped) devices";
    return on;
}
