#include "HrbConfigDispatcher.h"

#include "HrbAppConfig.h"
#include "base/DeviceId.h"
#include "base/DeviceIndexEnabled.h"

HrbConfigDispatcher::HrbConfigDispatcher(QObject *parent)
    : BaseConfigDispatcher(parent)
    , cache(new HrbAppConfig())
{}

DeviceIndexEnabledMap HrbConfigDispatcher::getDeviceIndexEnabledMap() const
{
    return cache->getDeviceIndexEnabledMap();
}

void HrbConfigDispatcher::setConfig(const HrbAppConfig &newConfig)
{
    if (newConfig == *cache)
        return;

    *cache = newConfig;
    incrementConfigKey(true);
}

void HrbConfigDispatcher::setDeviceList(const DeviceIndexEnabledMap &map)
{
    if (getDeviceIndexEnabledMap() == map)
        return;

    DeviceIndexSet oldSet = DeviceIndexSet(getDeviceIndexEnabledMap());
    DeviceIndexSet newSet = DeviceIndexSet(map);

    const auto toAdd = newSet - oldSet;
    const auto toDel = oldSet - newSet;
    const auto remain = oldSet & newSet;

    for(auto i : toDel){
        cache->hrb.remove(i);
        cache->ttb.remove(i);
    }
    for(auto i : remain){
        auto hrbIt = cache->hrb.find(i);
        if(hrbIt != cache->hrb.end())
            hrbIt->enabled = map[i].enabled;

        auto ttbIt = cache->ttb.find(i);
        if(ttbIt != cache->ttb.end())
            ttbIt->enabled = map[i].enabled;
    }

    for(auto i : toAdd){
        if(i.getDevId() == DEVICE_ID_HRB6ASD) {
            auto &cfg = cache->hrb[i];
            cfg.index = i;
            cfg.enabled = map[i].enabled;
        }

        if(i.getDevId() == DEVICE_ID_TTB9V) {
            auto &cfg = cache->ttb[i];
            cfg.index = i;
            cfg.enabled = map[i].enabled;
        }
    }
    incrementConfigKey();
    emit deviceListChanged(getDeviceIndexEnabledMap());
}

void HrbConfigDispatcher::setTriggerSetup(const HrbTriggerSetup &cfg)
{
    if (cache->triggerSetup == cfg)
        return;

    cache->triggerSetup = cfg;
    incrementConfigKey();
}

void HrbConfigDispatcher::setHrbDeviceConfig(const ModularDeviceConfig &cfg)
{
    if (!cache->ttb.contains(cfg.index))
        return;
    auto &hrbCfg = cache->hrb[cfg.index];
    if (hrbCfg == cfg)
        return;

    hrbCfg = cfg;
    incrementConfigKey();
}

void HrbConfigDispatcher::setTtbDeviceConfig(const ModularDeviceConfig &cfg)
{
    if (!cache->ttb.contains(cfg.index))
        return;
    auto &ttbCfg = cache->ttb[cfg.index];
    if (ttbCfg == cfg)
        return;

    ttbCfg = cfg;
    incrementConfigKey();
}
