#ifndef CLIENT_H
#define CLIENT_H

#include <QObject>

#include "EvBConfig.h"
#include "ClientFsm.h"

template<class T> class RingBuffer;
class TcpClient;
class TcpStatistic;
class Analizer;
struct EventMetaInfo;
class QHostAddress;
class QSemaphore;
class AnalyzerStat;
struct EventSizePeriod;
struct RcRunId;

class Client : public ClientFsm
{
    Q_OBJECT
public:
    explicit Client(const EvBClientConfig &cc,int evNumIndex, QObject *parent);
    virtual ~Client() override;
    Analizer *getAnalizer() const { return analizer; }

signals:
    void readOutEnChanged(bool en) const;
    void stopReadOut();
    void clientStateUpdated(ClientIndex index, FsmState state); // from combiner and clients
    void requestState(FsmState state) const;
    void destroyingClient();
    void runError();
    void statUpdated(const AnalyzerStat &) const;
    void evSizePeriodUpdated(const ClientIndex &i, const EventSizePeriod &s) const;
    void reqPrintFsmInput() const; // proxy signal to connect this, anal and tcp

public slots:
    void destroyClient(const ClientIndex &i);
    void updateClientReadout(const ClientIndex &i, bool readout);
    void setClientRequestState(const FsmState &st);

private slots:
    void onFsmStateChanged(const FsmState &cur);

protected:
    void doReqStateActions(const FsmState &st) final;

private:
    ClientIndex index;
    bool enabled = true;
    RingBuffer<char> * const dataRingBuff;
    RingBuffer<EventMetaInfo> * const evRingBuff;
    QThread * const tcpClientThread;
    QThread * const analizerThread;
    TcpClient * const tcpClient;
    Analizer * analizer;
};

#endif // CLIENT_H
