#include "ClientManagerFsm.h"
#include  "base/ClientIndexEnabled.h"

#include <QLoggingCategory>

namespace {
Q_DECLARE_LOGGING_CATEGORY(fsm)
Q_LOGGING_CATEGORY(fsm, "afi.fsm.evb.ClientManager")
}

ClientManagerFsm::ClientManagerFsm(QObject *parent)
    : BaseEvbClientFsm(parent)
{
    setObjectName("ClientManagerFsm");
    recreateInput(new ClientManagerFsmInput());
}

void ClientManagerFsm::setClientList(const ClientIndexEnabledMap &newList)
{
    getInput()->clientsSet = newList.getEnabled();
    auto &clientsStates = getInput()->clientsStates;

    for(auto i : clientsStates.keys()) {
        if(!newList.contains(i))
            clientsStates.remove(i);
    }

    evaluate();
}

void ClientManagerFsm::setClientState(ClientIndex index, FsmState state)
{
    getInput()->clientsStates[index] = state;
    evaluate();
}

void ClientManagerFsm::printInputString(QDebug dbg) const
{
    dbg << *getInput();
}

bool ClientManagerFsm::checkSubStates(const FsmState &state) const
{
    const auto &i = getInput();
    for(ClientIndex index : i->clientsSet){
        if(state != i->clientsStates.value(index, FsmState::Unknown))
            return false;
    }
    return true;
}

bool ClientManagerFsm::hasAnyError() const
{
    const auto &i = getInput();
    for(ClientIndex index : i->clientsSet) {
        if(i->clientsStates.value(index, FsmState::Unknown) == FsmState::Error)
            return true;
    }
    return false;
}

QDebug operator<<(QDebug dbg, const ClientManagerFsmInput &in)
{
    dbg << BaseFsmInput(in);
    dbg << "clientsSet:" << in.clientsSet << "; ";
    dbg << "clientsStates:" << in.clientsStates << "; ";
    return dbg;
}
