#ifndef TluTtlIoWidget_H
#define TluTtlIoWidget_H

#include <QWidget>

#include "base/DeviceIndex.h"
#include "device-modules/TluTtlIoControl/TluTtlIoControlConfig.h"
#include "fsm/FsmState.h"
#include "lib-common/BaseDeviceAppConfig.h"

class QGridLayout;
class QLabel;
class QComboBox;
class ConfigKey;

class SingleTtlWidget : public QWidget
{
    Q_OBJECT
public:
    SingleTtlWidget(int id, QWidget *parent = nullptr);
    ~SingleTtlWidget();
    void setRegConfig(const reg_tlu_ttl_io_t &regCfg);

    int id;
    QLabel * const lab;
    QComboBox * const cb;
};

class TluTtlIoWidget : public QWidget
{
    Q_OBJECT

public:
    explicit TluTtlIoWidget(QWidget *parent = nullptr);
    ~TluTtlIoWidget();
    void setDeviceIdFilter(DeviceId id) { deviceIdFilter = id; }

public slots:
    void setTtlIoStartupCfg(const DeviceIndex &, const TluTtlIoControlConfig &cfg);
    void onFsmStateChange(const FsmState &state);
    virtual void setConfig(const ConfigKey &, const BaseDeviceAppConfig &_config);

private slots:
    void ttlItemChanged(int index);

signals:
    void configChanged(const BaseDeviceAppConfig &) const;

private:
    void addTtl(reg_tlu_ttl_io_t r);
    void removeTtl();
    int getChNum() const;
    void unifyConfigs();
    void refreshGui();
    void applyCfgChange();
    bool isSkippedDevice(const DeviceIndex &i) const;

    QGridLayout * const gridLayout;
    QVector<SingleTtlWidget*> singleTtlWidgets;
    QMap<DeviceIndex, TluTtlIoControlConfig> startupConfigs;
    BaseDeviceAppConfig config;
    TluTtlIoControlConfig curTtlCfg;
    DeviceId deviceIdFilter;
};

#endif // TluTtlIoWidget_H
