/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef VMETRIGSMODEL_H
#define VMETRIGSMODEL_H

#include <QAbstractTableModel>

class DeviceIndex;
struct LldpInfo;
class LldpInfoCache;
struct VmeTrigConfig;
struct VmeTrigStatus;
class VmeTrigTable;
struct VmeTrigTableEntry;

class VmeTrigsModel : public QAbstractTableModel
{
    Q_OBJECT
public:

    enum Columns {COL_ENABLE, COL_VME_ID, COL_VME_SERIAL, COL_VME_CHECK_EV_NUM, COL_VME_HOST,
                   COL_VME_TEMP, COL_VME_EV_NUM,
                   COL_VME_CHECK_WR, COL_VME_WR_TIME_VALID, COL_VME_WR_SYNC_LOST, COL_VME_WR_LINK_ERROR,
                   COL_VME_BUSY,COL_VME_IDLE,COL_VME_OCCUPANCY,
                   COL_VME_TOTAL
    };

    VmeTrigsModel(QObject *parent = nullptr);
    ~VmeTrigsModel();

    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    virtual int columnCount(const QModelIndex &parent = QModelIndex()) const override;

    Qt::ItemFlags flags(const QModelIndex & index) const override;
    virtual QVariant data(const QModelIndex &index, int role) const override;
    virtual QVariant headerData(int section, Qt::Orientation orientation,
                                int role = Qt::DisplayRole) const override;
    bool setData(const QModelIndex& index, const QVariant& value, int role) override;

signals:
    void trigConfigChanged(const DeviceIndex &index, const VmeTrigConfig &config);

public slots:
    void lldpCacheUpdated(const LldpInfoCache &lldp);
    void setTrigConfig(const DeviceIndex &index, const VmeTrigConfig &config);
    void setTrigStatus(const DeviceIndex &index, const VmeTrigStatus &status);
    void addDevice(const DeviceIndex &index);
    void removeDevice(const DeviceIndex &index);
    void setEnabled(bool en);

private:
    void invalidateRow(const DeviceIndex &index);
    QVariant data_BackgroundRole(const DeviceIndex &index, int column) const;
    QVariant data_CheckStateRole(const DeviceIndex &index, int column) const;
    QVariant data_DisplayRole(const DeviceIndex &index, int column) const;

    LldpInfoCache * const lldpInfoCache;
    VmeTrigTable * const devices;
    bool enabled = true;
};

#endif // VMETRIGSMODEL_H
