#ifndef TQDCMAINWINDOW_H
#define TQDCMAINWINDOW_H

#include "TqdcStatus.h"
#include "lib-common/ReadoutAppMainWindow.h"

namespace Ui {
class TqdcMainWindow;
}

class CalibrationDialog;
class ConfigKey;
class DeviceChModel;
class DeviceChTreeView;
class FirFilterWidget;
class QTimer;
class RunLogicWidget;
class TrigCtrlStatWidget;
class TtlIoWidget;
class WaveBlcWidget;

class TqdcMainWindow : public ReadoutAppMainWindow
{
    Q_OBJECT

public:
    explicit TqdcMainWindow(QWidget *parent = nullptr);
    ~TqdcMainWindow() override;

    DeviceStatusTableViewFancy *getDeviceStatusTableView() override;
    DeviceChModel * getDeviceTreeModel() const
    { return treeModel; }
    DeviceChTreeView * getDeviceChTreeView() const;
    FirFilterWidget *getFirFilterWidget() const;
    RunLogicWidget *getRunLogicWidget() const;
    TtlIoWidget *getWidgetTtl() const;
    TrigCtrlStatWidget *getTrigCtrlStatWidget() const;
    WaveBlcWidget *getWaveBlcWidget() const;

signals:
    void closeAll();
    void softReset();
    void requestInlCorr(const DeviceIndex index);
    void prepareForStop(quint32 stopEvNum);
    void mstream_reset_all(bool doStatClear);
    void sendRemoteState(QString state, bool ready4Run=false);
    void performLocalStart() const;
    void performLocalStop() const;
    void printFsmInputs() const;

public slots:
    void setConfig(const ConfigKey &config_key, const BaseDeviceAppConfig &newConfig) override;
    void deviceInfoUpdated(DeviceIndex index, int maxSampleCount);
    void deviceStatusUpdated(const TqdcStatus &status);

private slots:
    void on_pushButtonStart_clicked();
    void on_pushButtonStop_clicked();
    void on_radioButtonGain1_toggled(bool checked);
    void on_radioButtonGain4_toggled(bool checked);
    void on_comboBoxZsMode_currentIndexChanged(int index);
    void on_spinBoxLatency_valueChanged(int lat);
    void on_spinBoxMatchWin_valueChanged(int win);
    void on_spinBoxAdcWin_valueChanged(int val);
    void on_spinBoxAdcPreWin_valueChanged(int preWin);
    void on_actionCalibrationData_triggered();
    void on_actionSeparate_channel_enable_toggled(bool checked);
    void on_actionDisable_match_win_limitations_toggled(bool checked);
    void on_actionReset_MStream_triggered();
    void on_actionSend_soft_reset_triggered();
    void on_actionPrint_FSM_inputs_triggered();
    void on_actionTxt_data_format_triggered();
    void on_actionRaw_data_format_triggered();
    void on_actionAdd_HPTDC_event_header_toggled(bool arg1);
    void on_actionAdd_HPTDC_event_tralier_toggled(bool checked);
    void on_comboBoxEdges_currentIndexChanged(int index);

protected:
    virtual void saveAppGuiSettings(QSettings &s) const override;
    virtual void loadAppGuiSettings(const QSettings &s) override;
    void updateControlState() override;

private:
    void init_main_window();
    void copyAndSetupDevice();
    void setupDevice();
    void updateGuiVal();
    void controlStateEnable(bool enable);
    void deviceStatusUpdatedRequest();
    void deviceStatusUpdatedResponce();
    bool isControlBlocked() const;
    void updateMatchWinMaximum() const;
    void updateWdgetWarnings() const;

public:
    CalibrationDialog * const calibrationDialog;

private:
    Ui::TqdcMainWindow *ui;
    QMap<DeviceIndex, TqdcStatus> dev_status;
    int blockControl = 0;

    quint32 stopEvNum = 0;
    DeviceChModel * const treeModel;
    QTimer * const status_update_timer;
};

#endif // TQDCMAINWINDOW_H
