#include "TrcSpinBoxDelegate.h"

#include <QDebug>
#include <QTimer>
#include <QSpinBox>

#include "TrcTreeColumns.h"

namespace {
void setSpinBoxMaximum(QSpinBox *spin, const QModelIndex &index) {
    int maxValue = index.model()->data(index, Qt::UserRole+1).toInt();
    spin->setMaximum(maxValue);
}
void setSpinBoxEnabled(QSpinBox *spin, const QModelIndex &index) {
    bool en = index.model()->data(index, Qt::UserRole+5).toBool();
    spin->setEnabled(en);
}
void setSpinBoxValue(QSpinBox *spin, const QModelIndex &index) {
    int value_int = index.model()->data(index, Qt::EditRole).toInt();
    spin->setValue(value_int);
}
}
TrcSpinBoxDelegate::TrcSpinBoxDelegate(QObject *parent)
    : QItemDelegate(parent)
    , commitDataTimer(new QTimer(this))
{
    commitDataTimer->setInterval(1);
    commitDataTimer->setSingleShot(true);
    connect(commitDataTimer, &QTimer::timeout,
            this, &TrcSpinBoxDelegate::flushCommitData);
}

QWidget *TrcSpinBoxDelegate::createEditor(QWidget *parent,
                                       const QStyleOptionViewItem &/* option */,
                                       const QModelIndex &index) const
{
     QSpinBox *spin = new QSpinBox(parent);
     setSpinBoxMaximum(spin, index);
     setSpinBoxEnabled(spin, index);
     setSpinBoxValue(spin, index);

     connect(spin, &QSpinBox::textChanged,
             this, &TrcSpinBoxDelegate::dataChanged);
     spin->setGroupSeparatorShown(true);

     return spin;
}

void TrcSpinBoxDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const
{
    QSpinBox *spin = static_cast<QSpinBox*>(editor);
    spin->blockSignals(true);

    if(index.column() == static_cast<int>(TrcTreeColumns::COL_AFTER_TIME)) {
        setSpinBoxMaximum(spin, index);
    }

    setSpinBoxEnabled(spin, index);
    setSpinBoxValue(spin, index);

    spin->blockSignals(false);
}

void TrcSpinBoxDelegate::setModelData(QWidget *editor,
                                   QAbstractItemModel *model,
                                   const QModelIndex &index) const
{
    QSpinBox *spin_box = static_cast<QSpinBox*>(editor);
    spin_box->interpretText();
    int value = spin_box->value();
    if(model->data(index, Qt::EditRole).toInt() != value)
        model->setData(index, value, Qt::EditRole);
}

void TrcSpinBoxDelegate::updateEditorGeometry(QWidget *editor,
                                           const QStyleOptionViewItem &option,
                                           const QModelIndex &/* index */) const
{
    editor->setGeometry(option.rect);
}

void TrcSpinBoxDelegate::dataChanged()
{
    if(commitDataWidget) {
        if(commitDataWidget == sender())
            return;
        flushCommitData();
    }
    commitDataWidget = sender();
    commitDataTimer->start();
}

void TrcSpinBoxDelegate::flushCommitData()
{
    if(!commitDataWidget)
        return;
    commitData(qobject_cast<QWidget*>(commitDataWidget));
    commitDataWidget = nullptr;
}
