#include "ConfigDocumentConverter.h"

#include "ConfigMetaConverter.h"
#include "ConfigOidConverter.h"
#include "ConfigSelectorConverter.h"

namespace {
inline constexpr const char *k_id = "_id";
}

template<>
bool AbstractConfigConverter<ConfigDocument>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<ConfigDocument>::toJsonObject(const ConfigDocument &in)
{
    QVariantMap m;
    if(in.isEmpty()){
        m = in.getConfig().toVariantMap();
    }
    else{
        auto oid = in.getConfigOid();
        if(!oid.isEmpty())
            m[k_id] = QVariant::fromValue(oid);
        m[k_config_selector] = QVariant::fromValue(in.getConfigSelector());
        m[k_meta] = QVariant::fromValue(in.getConfigMeta());
        auto cfg = in.getConfig();
        if(!cfg.isEmpty())
            m[k_config] = in.getConfig();
    }
    return fromVariantMap(m);
}

template<>
ConfigDocument AbstractConfigConverter<ConfigDocument>::fromJsonObject(const QJsonObject &json)
{
    ConfigSelector cs;
    update_value(json, k_config_selector, &cs);
    if(cs.isEmpty()){
        ConfigDocument to_return(json);
        return to_return;
    }
    else{
        ConfigDocument to_return(
            QVariant::fromValue(json[k_id]).value<ConfigOid>(),
            cs,
            QVariant::fromValue(json[k_meta]).value<ConfigMeta>(),
            json[k_config].toObject()
        );
        return to_return;
    }
}

template<>
QJsonValue AbstractConfigConverter<ConfigDocument>::toJsonValue(const ConfigDocument &in)
{
    return toJsonObject(in);
}

template<>
ConfigDocument AbstractConfigConverter<ConfigDocument>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<ConfigDocument>::registerSubTypes()
{
    AbstractConfigConverter<ConfigOid>::registerTypes();
    AbstractConfigConverter<ConfigSelector>::registerTypes();
    AbstractConfigConverter<ConfigMeta>::registerTypes();
}

