/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef GLOBALS_H
#define GLOBALS_H

#include <QString>
#include <memory>

#include "ConfigSelector.h"
#include "DaqModule.h"

class QSettings;

class Globals
{
public:
    static Globals &instance() {
        static Globals obj;
        return obj;
    }
    static std::shared_ptr<const QSettings> getSettings();
    static QString getAppDomain();

    static DaqModule getDaqModule() {
        return instance().daqModule;
    }
    static DaqModule getDaqModuleType() {
        return instance().daqModule;
    }
    static void setDaqModuleType(const DaqModule::Type &value) {
        instance().daqModule = DaqModule(value);
    }
    static void setDaqModule(const DaqModule &value) {
        instance().daqModule = value;
    }

    static ProgramIndex getInitialProgramIndex() {
        return instance().initialProgramIndex;
    }
    static void setInitialProgramIndex(const ProgramIndex &value) {
        instance().initialProgramIndex = value;
    }

    static QString getInitialConfigurationName() {
        return instance().initialConfigurationName;
    }
    static void setInitialConfigurationName(const QString &value) {
        instance().initialConfigurationName = value;
    }

    static ConfigSelector getInitialConfigSelector();

    static void setCommonConfPath(const QString &value) {
        instance().commonConfPath = value;
    }

    static bool getDebugMode() {
        return instance().debug_mode;
    }
    static void setDebugMode(bool value) {
        instance().debug_mode = value;
    }

    static bool getUseQtDialogs() {
        return instance().useQtDialogs;
    }
    static void setUseQtDialogs(bool value) {
        instance().useQtDialogs = value;
    }

    static QString getEnvironment() {
        return instance().environment;
    }
    static void setEnvironment(const QString &value) {
        instance().environment = value;
    }
    static constexpr const char *organizationName = "AFI Electronics";
    static constexpr const char *organizationDomain = "local";
    static constexpr const char *common_settings_app_name = "Common";
private:
    bool debug_mode = false;
    QString environment = "production";
    QString commonConfPath = "/etc/afi-daq/common.conf";
    DaqModule daqModule;
    ProgramIndex initialProgramIndex;
    QString initialConfigurationName;
    bool useQtDialogs = true;
};

#endif // GLOBALS_H
