/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "Ipv4Prefix.h"

Ipv4Prefix Ipv4Prefix::parse(const QString &s)
{
    quint32 ip = 0;
    int maskbits = 32;
    // 0.0.0.0/0
    if (s.contains("/")) {
        const auto &fields = s.split("/");
        if (s.size() >= 1) {
            bool ok;
            auto u = QHostAddress(fields[0]).toIPv4Address(&ok);
            if (ok)
                ip = u;
            if (s.size() >= 2) {
                auto v = fields[1].toUShort(&ok);
                if (ok && v <= 32)
                    maskbits = v;
            }
        }
    }
    return Ipv4Prefix(ip, maskbits);
}

Ipv4Prefix::operator QString() const
{
    return QHostAddress(_ip).toString() + "/" + QString::number(_maskbits);// + " " + QHostAddress(mask).toString();
}

bool Ipv4PrefixList::match(quint32 ip) const
{
    if (list.contains(Ipv4Prefix(0,0)))
        return true;
    Ipv4Prefix result;
    for (auto p : list) {
        if ((ip & p.mask()) == (p.ip() & p.mask()))
        return true;
    }
    return false;
}

Ipv4Prefix Ipv4PrefixList::findLongestMatch(quint32 ip, bool *found) const
{
    int max_bits = 0;
    Ipv4Prefix result;
    for (auto p : list) {
        if ((ip & p.mask()) == (p.ip() & p.mask())) {
            auto bits = p.maskbits();
            if (bits > max_bits) {
                max_bits = bits;
                result = p;
            }
        }
    }
    if (found)
        *found = max_bits > 0;
    return result;
}

QDebug operator<<(QDebug dbg, const Ipv4PrefixList &r)
{
    dbg << r.list;
    return dbg;
}
