/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef CONFIGSELECTOR_H
#define CONFIGSELECTOR_H

#include <QMetaType>
#include <QString>

#include "ConfigIndexName.h"
#include "ClientIndex.h"
#include "DaqModule.h"
#include "ProgramIndex.h"

class RcRunId;

class ConfigSelector {
public:
    friend class ConfigSelectorConverter;

    ConfigSelector() {}

    ConfigSelector(const DaqModule &module)
        :
        daqModule(module)
    { }

    ConfigSelector(const DaqModule &module,
                   const ConfigIndexName &conf)
        :
        daqModule(module),
        configIndexName(conf)
    { }


    [[deprecated("Use ConfigSelector(DaqModule, ConfigIndexName)")]]
    ConfigSelector(const DaqModule &module,
                   const ProgramIndex &_index,
                   const ConfigurationName _name)
        :
        daqModule(module),
        configIndexName(_index, _name)
    { }
    bool isEmpty() const {return daqModule.isEmpty(); }
    ProgramIndex getIndex() const {
        return configIndexName.program_index;
    }
    [[deprecated("Use operator=()")]]
    void setIndex(const ProgramIndex &index)
    {
        configIndexName.program_index = index.isEmpty() ? ProgramIndex() : index;
    }
    DaqModule getType() const { return daqModule; }
    ConfigIndexName getConfigIndexName() const { return configIndexName; }
    ConfigurationName getConfigurationName() const {
        return configIndexName.configuration_name;
    }
    [[deprecated("Use operator=()")]]
    void setConfigurationName(const ConfigurationName &s)
    {
        configIndexName.configuration_name = s.isEmpty() ? ConfigurationName() : s;
    }
    ClientIndex getClientIndex() const { return {getType(), getIndex()}; }
    QString toWindowTitle() const;
    QString toString() const;
    // QString makeConfigRootName(const RcRunId &runId) const;

    friend bool operator==(const ConfigSelector &l, const ConfigSelector &r) = default;
    friend bool operator<(const ConfigSelector &l, const ConfigSelector &r);
    friend QDebug operator<<(QDebug dbg, const ConfigSelector &r);

private:
    DaqModule daqModule;
    ConfigIndexName configIndexName;
};

Q_DECLARE_METATYPE(ConfigSelector)

inline QDataStream &operator<<(QDataStream &out, const ConfigSelector &pod) {
    out<< pod.getType() << pod.getConfigIndexName();
    return out;
}

inline QDataStream &operator>>(QDataStream &in, ConfigSelector &pod) {
    QString type;
    ConfigIndexName cin;
    in >> type >> cin;
    pod = ConfigSelector(DaqModule::fromString(type), cin);
    return in;
}

#endif // CONFIGSELECTOR_H
