/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef DEVICEIDENTITYPROXYMODEL_H
#define DEVICEIDENTITYPROXYMODEL_H

#include <QIdentityProxyModel>
#include <QMap>

#include "base/DnsLookup.h"

class DeviceIndexEnabled;

class DeviceIdentityProxyModel : public QIdentityProxyModel
{
    Q_OBJECT
public:
    DeviceIdentityProxyModel(QObject* parent = nullptr);
    QVariant data(const QModelIndex &index, int role) const override;
    Qt::ItemFlags flags(const QModelIndex & index) const override;
    virtual QVariant headerData(int section, Qt::Orientation orientation,
                                int role = Qt::DisplayRole) const override;

private:
    QVariant getDisplayData(const QModelIndex &index) const;
    QVariant getToolTipData(const QModelIndex &index) const;
    QVariant getDecorationData(const QModelIndex &index) const;

    QMap<quint32, QString> getHostMap() const;
    //QString getIpStr(bool locked, quint32 ip, quint16 port) const;
    void reverseMapUpdated(const DnsReverseMap &map);

    DnsLookup * const dnsLookup;
    DnsReverseMap resolved_hosts;
};

#endif // DEVICEIDENTITYPROXYMODEL_H
