/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "DeviceIndexFilterProxyModel.h"

#include <QDebug>

#include "BaseDeviceTableModel.h"

DeviceIndexFilterProxyModel::DeviceIndexFilterProxyModel(QObject *parent)
    : QSortFilterProxyModel(parent)
{

}

void DeviceIndexFilterProxyModel::setInverseMode(bool inverse)
{
    this->inverse = inverse;
    invalidateFilter();
}

void DeviceIndexFilterProxyModel::setFilter(const DeviceIndexSet &r)
{
    filter = r;
    invalidateFilter();
}

bool DeviceIndexFilterProxyModel::filterAcceptsRow(
        int sourceRow, const QModelIndex &sourceParent) const
{
    auto sourceModel = this->sourceModel();
    Q_ASSERT(sourceModel);
    if (!sourceModel)
        return true;
    const QModelIndex &indexDeviceType = sourceModel->index(
                sourceRow, BaseDeviceTableModel::COL_DEVICE_ID, sourceParent);
    const QModelIndex &indexDeviceSerial = sourceModel->index(
                sourceRow, BaseDeviceTableModel::COL_SERIAL, sourceParent);
    bool ok;
    const auto &device_id = sourceModel->data(indexDeviceType, Qt::UserRole).toUInt(&ok);
    Q_ASSERT(ok);
    if (!ok) {
        return true;
    }
    const auto &serial_id = sourceModel->data(indexDeviceSerial, Qt::UserRole).toULongLong(&ok);
    Q_ASSERT(ok);
    if (!ok)
        return true;
    DeviceIndex deviceIndex(static_cast<DeviceId>(device_id), serial_id);
    return inverse ^ filter.contains(deviceIndex);
}
