/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "LldpDiscovery.h"

#include "LldpDiscovery_p.h"

LldpDiscovery::LldpDiscovery()
    : QObject(nullptr),
    d_ptr(new LldpDiscoveryPrivate(this))
{
    Q_D(LldpDiscovery);
    d->q_ptr = this;

    setObjectName("LldpDiscovery");

    qRegisterMetaType<LldpInfoCache>();

    connect(d, &LldpDiscoveryPrivate::cacheUpdated,
            this, &LldpDiscovery::cacheUpdated);

    connect(d, &LldpDiscoveryPrivate::deviceStateChanged,
            this, &LldpDiscovery::deviceStateChanged);

    connect(this, &LldpDiscovery::setDeviceIdFilter,
            d, &LldpDiscoveryPrivate::setDeviceIdFilter);
}

LldpDiscovery::~LldpDiscovery()
{
   delete d_ptr;
}

LldpInfo LldpDiscovery::lookup_internal(const DeviceIndex &index, bool *Ok) const
{
    Q_D(const LldpDiscovery);
    bool found = d->map.contains(index);
    if (Ok)
        *Ok = found;
    if (found)
        return d->map.value(index);
    return LldpInfo();
}
