#include "MscHistConfigConverter.h"
#include "MscHistConfig.h"
#include "base/ConfigConverterUtil.h"

namespace {
    constexpr const char *k_logTypeIsLen = "logTypeIsLen";

    constexpr const char *k_name = "name";
    constexpr const char *k_enable = "enable";
    constexpr const char *k_plot_slice = "plot_slice";

    constexpr const char *k_log = "log";
    constexpr const char *k_plot = "plot";
}


// ================ MscHistLogConfig
template<>
bool AbstractConfigConverter<MscHistLogConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<MscHistLogConfig>::toJsonObject(const MscHistLogConfig &in)
{
    QVariantMap m;
    m[k_logTypeIsLen] = QVariant::fromValue(in.logTypeIsLen);
    return fromVariantMap(m);
}

template<>
MscHistLogConfig AbstractConfigConverter<MscHistLogConfig>::fromJsonObject(const QJsonObject &in)
{
    MscHistLogConfig out;
    update_value(in, k_logTypeIsLen, &out.logTypeIsLen);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<MscHistLogConfig>::toJsonValue(const MscHistLogConfig &in)
{
    return toJsonObject(in);
}

template<>
MscHistLogConfig AbstractConfigConverter<MscHistLogConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<MscHistLogConfig>::registerSubTypes()
{}


// ================ MscHistPlotConfig
template<>
bool AbstractConfigConverter<MscHistPlotConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<MscHistPlotConfig>::toJsonObject(const MscHistPlotConfig &in)
{
    QVariantMap m;
    m[k_name] = QVariant::fromValue(in.name);
    m[k_enable] = QVariant::fromValue(in.enable);
    m[k_plot_slice] = QVariant::fromValue(in.plotSlice);
    return fromVariantMap(m);
}

template<>
MscHistPlotConfig AbstractConfigConverter<MscHistPlotConfig>::fromJsonObject(const QJsonObject &in)
{
    MscHistPlotConfig out;
    update_value(in, k_name, &out.name);
    update_value(in, k_enable, &out.enable);
    update_value(in, k_plot_slice, &out.plotSlice);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<MscHistPlotConfig>::toJsonValue(const MscHistPlotConfig &in)
{
    return toJsonObject(in);
}

template<>
MscHistPlotConfig AbstractConfigConverter<MscHistPlotConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<MscHistPlotConfig>::registerSubTypes()
{}


// ================ MscHistConfig
template<>
bool AbstractConfigConverter<MscHistConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<MscHistConfig>::toJsonObject(const MscHistConfig &in)
{
    QVariantMap m;

    m[k_log] = ConfigConverterUtil::toJsonValue(in.log);
    m[k_plot] = ConfigConverterUtil::toJsonValue(in.plot);

    return fromVariantMap(m);
}

template<>
MscHistConfig AbstractConfigConverter<MscHistConfig>::fromJsonObject(const QJsonObject &in)
{
    MscHistConfig out;
    if(in.contains(k_log))
        out.log = ConfigConverterUtil::toMap<int, MscHistLogConfig>(in[k_log]);
    if(in.contains(k_plot))
        out.plot = ConfigConverterUtil::toMap<int, MscHistPlotConfig>(in[k_plot]);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<MscHistConfig>::toJsonValue(const MscHistConfig &in)
{
    return toJsonObject(in);
}

template<>
MscHistConfig AbstractConfigConverter<MscHistConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<MscHistConfig>::registerSubTypes()
{
    AbstractConfigConverter<MscHistLogConfig>::registerTypes();
    AbstractConfigConverter<MscHistPlotConfig>::registerTypes();
}
