/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef NETWORKPORTCONFIG_H
#define NETWORKPORTCONFIG_H

#include <stdint.h>
#include <algorithm>
#include <QMetaType>

struct NetworkPortConfig
{
    friend bool operator==(const NetworkPortConfig &, const NetworkPortConfig &) = default;
    static uint16_t getMtuMin() { return 576; }
    static uint16_t getMtuMax() { return 9216; }
    static uint32_t getDefragTimeoutMin() { return 0; }
    static uint32_t getDefragTimeoutMax() { return 1000000; }
    static uint32_t getRetxTimeoutMin() { return 10000; }
    static uint32_t getRetxTimeoutMax() { return 1000000; }
    void applyLimits() {
        mtu = std::clamp(mtu, getMtuMin(), getMtuMax());
        defragTimeoutUs = std::clamp(defragTimeoutUs, getDefragTimeoutMin(), getDefragTimeoutMax());
        retxTimeoutUs = std::clamp(retxTimeoutUs, getRetxTimeoutMin(), getRetxTimeoutMax());
    }

    bool mstreamReset = false;
    uint16_t mtu = 1500;
    uint32_t defragTimeoutUs = 10000;
    uint32_t retxTimeoutUs = 100000;
};

Q_DECLARE_METATYPE(NetworkPortConfig)

#endif // NETWORKPORTCONFIG_H
