#include "RcTrigConfigConverter.h"

#include "base/ConfigConverterUtil.h"

#include "RcTrigConfig.h"

namespace {
constexpr const char *k_xOffMask = "xOffMask";
constexpr const char *k_trigOpen = "trigOpen";
constexpr const char *k_trigSrc = "trigSrc";
constexpr const char *k_trigTimerFreq = "trigTimerFreq";
constexpr const char *k_trigRandomFreq = "trigRandomFreq";
constexpr const char *k_trigRandomDeadTimeUs = "trigRandomDeadTimeUs";
}

template<>
bool AbstractConfigConverter<RcTrigConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<RcTrigConfig>::toJsonObject(const RcTrigConfig &in)
{
    QVariantMap m;

    m[k_xOffMask] = QVariant::fromValue(in.xOffMask);
    m[k_trigOpen] = QVariant::fromValue(in.trigOpen);
    m[k_trigSrc] = QVariant::fromValue(in.trigSrc);
    m[k_trigTimerFreq] = QVariant::fromValue(in.trigTimerFreq);
    m[k_trigRandomFreq] = QVariant::fromValue(in.trigRandomFreq);
    m[k_trigRandomDeadTimeUs] = QVariant::fromValue(in.trigRandomDeadTimeUs);

    return fromVariantMap(m);
}

template<>
RcTrigConfig AbstractConfigConverter<RcTrigConfig>::fromJsonObject(const QJsonObject &in)
{
    RcTrigConfig out;

    update_value(in, k_xOffMask, &out.xOffMask);
    update_value(in, k_trigOpen, &out.trigOpen);
    update_value(in, k_trigSrc, &out.trigSrc);
    update_value(in, k_trigTimerFreq, &out.trigTimerFreq);
    update_value(in, k_trigRandomFreq, &out.trigRandomFreq);
    update_value(in, k_trigRandomDeadTimeUs, &out.trigRandomDeadTimeUs);

    return out;
}

template<>
QJsonValue AbstractConfigConverter<RcTrigConfig>::toJsonValue(const RcTrigConfig &in)
{
    return toJsonObject(in);
}

template<>
RcTrigConfig AbstractConfigConverter<RcTrigConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<RcTrigConfig>::registerSubTypes()
{
}
