#ifndef TQDCCORECONFIG_H
#define TQDCCORECONFIG_H

#include "WaveBlc/WaveBlcConfig.h"
#include <QMetaType>
#include <QMap>

class WaveRecConfig;
class WaveTrigConfig;
struct TdcCoreConfig;
class BoardGpioConfig;
class HitCoreConfig;
enum DeviceId : uint8_t;

struct TqdcCoreConfig
{
    enum Gain {Gain1 = 0, Gain4 = 1};

    void setField(QMap<int, int> TqdcCoreConfig::*f, int thr);
    static bool getDefaultFieldVal(QMap<int, bool> TqdcCoreConfig::*f);
    static int getDefaultFieldVal(QMap<int, int> TqdcCoreConfig::*f);
    static bool hasTdc(DeviceId deviceId);
    static bool hasAdc(DeviceId deviceId);
    static bool hasInvert(DeviceId deviceId);
    static bool hasTrig(DeviceId deviceId);
    static int getNumberOfChannels(DeviceId deviceId);
    int getNCh() const { return nch; }
    void setNCh(int n);
    bool isGain4() const;
    int getSimilarField(QMap<int, int> TqdcCoreConfig::*p) const;
    void setGain(Gain value);
    void unify_config(const TqdcCoreConfig &ds) {
        TqdcCoreConfig newConfig(ds);
        newConfig.mergeLocalData(*this);
        *this = newConfig;
    }
    void makeTdcCoreConfig(TdcCoreConfig *tdc, DeviceId deviceId) const;
    void makeWaveRecConfig(WaveRecConfig *waveRec, DeviceId deviceId) const;
    void makeWaveTrigConfig(WaveTrigConfig *waveTrig) const;
    void makeBoardGpioConfig(BoardGpioConfig *boardGpio) const;
    void makeHitCoreConfig(HitCoreConfig *hitCore) const;
    void mergeLocalData(const TqdcCoreConfig &r)
    {
        chTdcEn = r.chTdcEn;
        chAdcEn = r.chAdcEn;
        chWfInvert = r.chWfInvert;
        chInv = r.chInv;
        chThr = r.chThr;
        chTrigEn = r.chTrigEn;
        chDigTrigEn = r.chDigTrigEn;
        chDigTrigRising = r.chDigTrigRising;
        chDigThr = r.chDigThr;
        chGain = r.chGain;

        chDigZsEn = r.chDigZsEn;
        chDigZsRising = r.chDigZsRising;
        chDigZsThr = r.chDigZsThr;

        nch = r.nch;
    }

    friend bool operator==(const TqdcCoreConfig &l, const TqdcCoreConfig &r) = default;

    int latency = 8; // in 24ns steps
    int matchWin = 8; // in 24ns steps
    bool hptdcStandby = false;  // not used
    bool invertInput = false; // not used
    bool invert_thr_trig = false; // not used
    bool invert_zs_thr = false; // not used

    WaveBlcChannelConfig blc;

    bool hptdcHdr = false;
    bool hptdcTrl = false;

    bool leadEdge = true;
    bool trailEdge = false;

    static const bool DEF_TDC_EN;
    QMap<int, bool> chTdcEn;
    static const bool DEF_ADC_EN;
    QMap<int, bool> chAdcEn;
    static const bool DEF_INV;
    QMap<int, bool> chWfInvert; // for AFI_SDB_WAVE_REC
    QMap<int, bool> chInv; // for AFI_SDB_Input_Hit_Core
    static const int DEF_THR;
    QMap<int, int> chThr; // anlg thr in mV
    static const bool DEF_TRIG_EN;
    QMap<int, bool> chTrigEn; // self-trig
    static const int DEF_DIG_THR;
    QMap<int, int> chDigThr; // dig thr
    static const bool DEF_DIG_TRIG_EN;
    QMap<int, bool> chDigTrigEn; // self-trig
    static const bool DEF_DIG_TRIG_RISING;
    QMap<int, bool> chDigTrigRising; // self-trig
    static const Gain DEF_GAIN;
    QMap<int, int> chGain;

    // AFI_SDB_WAVE_REC
    static const bool DEF_ZS_EN;
    QMap<int, bool> chDigZsEn;
    static const bool DEF_ZS_RISING;
    QMap<int, bool> chDigZsRising;
    static const int DEF_ZS_THR;
    QMap<int, int> chDigZsThr;

private:
    int nch = 0;
};
Q_DECLARE_METATYPE(TqdcCoreConfig)

#endif // TQDCCORECONFIG_H
