/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "TrigCtrlStatConfigConverter.h"
#include "TrigCtrlStat/TrigCtrlStatConfig.h"

namespace {
constexpr const char *k_trigTimerFreq = "trigTimerFreq";
constexpr const char *k_randomPulserFreq = "randomPulserFreq";
constexpr const char *k_randomPulserMinPeriod = "randomPulserMinPeriod";
constexpr const char *k_trigRandomDeadTime = "trigRandomDeadTime";
constexpr const char *k_analogThrDelay = "analogThrDelay";
constexpr const char *k_digThrDelay = "digThrDelay";
constexpr const char *k_source_mask = "source_mask";
} // namespace

template<>
bool AbstractConfigConverter<TrigCtrlStatConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<TrigCtrlStatConfig>::toJsonObject(const TrigCtrlStatConfig &in)
{
    QVariantMap m;

    m[k_trigTimerFreq] = QVariant::fromValue(in.trigTimerFreq);
    m[k_randomPulserFreq] = QVariant::fromValue(in.randomPulserFreq);
    m[k_randomPulserMinPeriod] = QVariant::fromValue(in.randomPulserMinPeriod);
    m[k_trigRandomDeadTime] = QVariant::fromValue(in.programmedDeadTime);
    m[k_analogThrDelay] = QVariant::fromValue(in.analogThrDelay);
    m[k_digThrDelay] = QVariant::fromValue(in.digThrDelay);

    m[k_source_mask] = QVariant::fromValue(in.source_mask);
    return fromVariantMap(m);
}

template<>
TrigCtrlStatConfig AbstractConfigConverter<TrigCtrlStatConfig>::fromJsonObject(const QJsonObject &in)
{
    TrigCtrlStatConfig out;

    update_value(in, k_trigTimerFreq, &out.trigTimerFreq);
    update_value(in, k_randomPulserFreq, &out.randomPulserFreq);
    update_value(in, k_randomPulserMinPeriod, &out.randomPulserMinPeriod);
    update_value(in, k_trigRandomDeadTime, &out.programmedDeadTime);
    update_value(in, k_analogThrDelay, &out.analogThrDelay);
    update_value(in, k_digThrDelay, &out.digThrDelay);

    update_value(in, k_source_mask, &out.source_mask);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<TrigCtrlStatConfig>::toJsonValue(const TrigCtrlStatConfig &in)
{
    return toJsonObject(in);
}

template<>
TrigCtrlStatConfig AbstractConfigConverter<TrigCtrlStatConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<TrigCtrlStatConfig>::registerSubTypes()
{
}
