/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "WaveRecStatus.h"

#include <QStringList>

static int bitCount(quint64 value)
{
    int count = 0;
    for (quint64 i=0; i<64; i++) {
        count += (value & 1);
        value >>= 1;
    }
    return count;
}

QString WaveRecStatus::getSummary() const
{
    int errors = bitCount(hfifo | mfifo | efifo);
    if (errors)
        return QString("%1 errors").arg(errors);
    else
        return "Ok";
}

QString WaveRecStatus::getFullText() const
{
    QStringList sl;
    if (hfifo)
        sl << QString("HFIFO: %1").arg(hfifo, 16, 16, QChar('0'));
    if (mfifo)
        sl << QString("MFIFO: %1").arg(mfifo, 16, 16, QChar('0'));
    if (efifo)
        sl << QString("EFIFO: %1").arg(efifo, 16, 16, QChar('0'));
    if (sl.empty())
        sl << "Ok";
    return sl.join("\n");
}

bool WaveRecStatus::hasErrors() const
{
    return hfifo || mfifo || efifo;
}
