/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef WAVETRIGCONFIG_H
#define WAVETRIGCONFIG_H

#include <QMap>
#include <QMetaType>

class WaveTrigChannelConfig
{
public:
    enum Edge {Rising = 0, Falling = 1};
    friend bool operator==(const WaveTrigChannelConfig &l, const WaveTrigChannelConfig &r) = default;

    Edge edge = Edge::Rising;
    qint16 trigLevel = 100;
    bool en = false;
};
Q_DECLARE_METATYPE(WaveTrigChannelConfig)

struct WaveTrigConfig
{
    friend bool operator==(const WaveTrigConfig &, const WaveTrigConfig &) = default;
    void mergeLocalData(const WaveTrigConfig &)
    {
        // nothing
    }
    WaveTrigChannelConfig getChannel(int channel) const
    {
        auto found = channels.find(channel);
        if (found == channels.end())
            return common;
        return *found;
    }
    void setChannel(int channel, const WaveTrigChannelConfig &channelConfig)
    {
        channels.insert(channel, channelConfig);
    }
    WaveTrigChannelConfig common;

private:
    QMap<int, WaveTrigChannelConfig> channels;
};
Q_DECLARE_METATYPE(WaveTrigConfig)

#endif // WAVETRIGCONFIG_H
