/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "FirFilterWidget.h"
#include "base/gui_utils.h"
#include "FirDesignDialog.h"
#include "FirFilter/SetFirFilterDialog.h"
#include "base/gui_utils.h"
#include "ui_FirFilterWidget.h"
#include "base/DeviceIndexEnabled.h"

#include <QSettings>


FirFilterWidget::FirFilterWidget(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::FirFilterWidget)
    , firDesignDialog(new FirDesignDialog(this))
{
    ui->setupUi(this);
    RecursiveWidgetSignalBlocker blocker(this);

    ui->comboBoxMode->addItem("Bypass", false);
    ui->comboBoxMode->addItem("Filter", true);

    ui->spinBoxTruncate->setRange(0, 3);
}

FirFilterWidget::~FirFilterWidget()
{
    delete ui;
}

void FirFilterWidget::setFirFilterConfig(const FirFilterConfig &newConfig)
{
    if (config == newConfig)
        return;
    config = newConfig;

    restoreGuiFromConfig();
}

void FirFilterWidget::setFirFilterBp(const DeviceIndex &index, const FirFilterBuildParams &bp)
{
    auto it = devBp.find(index);
    if(it != devBp.end() && *it == bp)
        return;

    devBp[index] = bp;
}

void FirFilterWidget::setDeviceList(const DeviceIndexEnabledMap &map)
{
    auto set = map.getEnabled();
    if(devSet == set)
        return;
    devSet = set;
}

void FirFilterWidget::restoreGuiFromConfig()
{
    RecursiveWidgetSignalBlocker blocker(this);

    setComboBoxCurrentItem(ui->comboBoxMode, config.en);
    ui->spinBoxTruncate->setValue(config.roundoff);
}

quint16 FirFilterWidget::getSizeLimit() const
{
    quint16 size = 32;
    for(auto i : devSet) {
        if(devBp.contains(i))
            size = std::min(size, devBp[i].size);
    }
    return size;
}

void FirFilterWidget::on_spinBoxTruncate_valueChanged(int arg1)
{
    config.roundoff = arg1;
    emit firFilterConfigChanged(config);
}

void FirFilterWidget::on_comboBoxMode_currentIndexChanged(int index)
{
    config.en = ui->comboBoxMode->itemData(index).toInt();
    emit firFilterConfigChanged(config);
}

void FirFilterWidget::on_pushButtonFirDesign_clicked()
{
    firDesignDialog->show();
}

void FirFilterWidget::on_pushButtonSetFir_clicked()
{
    SetFirFilterDialog d(config, this);
    auto size = getSizeLimit();
    d.setSizeLimit(size);
    if(d.exec())
        emit firFilterConfigChanged(d.getConfig());
}

