#include "SparseDialog.h"
#include "ui_SparseDialog.h"

#include "WaveRec/AdcBuildParams.h"
#include "base/DeviceIndexEnabled.h"
#include "base/gui_utils.h"

#include <optional>

SparseDialog::SparseDialog(QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::SparseDialog)
{
    ui->setupUi(this);
    connect(ui->checkBoxSSM, &QCheckBox::toggled,
            this, &SparseDialog::showSparseMarkersUpdated);
    ui->labelPeriodPointsSelection->hide();
    ui->comboBoxPointsSel->hide();
}

SparseDialog::~SparseDialog()
{
    delete ui;
}

void SparseDialog::setDeviceList(const DeviceIndexEnabledMap &new_devices)
{
    auto newEn = new_devices.getEnabled();
    if(devSet == newEn)
        return;
    devSet = newEn;
    upMaxSampleCnt();
}

void SparseDialog::deviceInfoUpdated(DeviceIndex index, const AdcBuildParams &par)
{
    setOffsetMaximum(index, par.adcMaxMatchWin);
    blockOffset(index, true);
}

void SparseDialog::setSparseParams(const SparseParams &sparseParams)
{
    if(cfg == sparseParams)
        return;
    cfg = sparseParams;
    restoreFormSettings();
}

void SparseDialog::setOffsetMaximum(const DeviceIndex &index, quint16 max)
{
    RecursiveWidgetSignalBlocker blocker(this);

     auto it = devMaxSampleCnt.find(index);
     if(it != devMaxSampleCnt.end() && *it == max)
         return;
     devMaxSampleCnt[index] = max;
     upMaxSampleCnt();
}

void SparseDialog::blockOffset(const DeviceIndex &index, bool blockOffset)
{
    if (blockOffset)
        devBlockOffset.insert(index);
    else
        devBlockOffset.remove(index);
}

void SparseDialog::upMaxSampleCnt() const
{
    std::optional<quint16> max;
    for(auto i : devSet) {
        auto it = devMaxSampleCnt.constFind(i);
        if(it == devMaxSampleCnt.cend())
            continue;
        max = std::max(*it, max.value_or(*it));
    }
    ui->spinBoxReadCellNum->setMaximum(max.value_or(cfg.readCellNum));
}

void SparseDialog::restoreFormSettings()
{
    RecursiveWidgetSignalBlocker blocker(this);

    ui->checkBoxEnable->setChecked(cfg.en);

    upMaxSampleCnt();
    ui->spinBoxReadCellNum->setValue(cfg.readCellNum);
    ui->spinBoxPeriod->setValue(cfg.period);
    ui->comboBoxPointsSel->setCurrentIndex(cfg.pointNumber);
}


void SparseDialog::on_checkBoxEnable_toggled(bool checked)
{
    cfg.en = checked;
    emit sparseParamsChanged(cfg);
}

void SparseDialog::on_spinBoxReadCellNum_valueChanged(int arg1)
{
    cfg.readCellNum = arg1;
    emit sparseParamsChanged(cfg);
}

void SparseDialog::on_spinBoxPeriod_valueChanged(int arg1)
{
    cfg.period = arg1;
    emit sparseParamsChanged(cfg);
}

void SparseDialog::on_comboBoxPointsSel_currentIndexChanged(int index)
{
    cfg.pointNumber = index;
    emit sparseParamsChanged(cfg);
}
