#ifndef SPILLCONTROLWIDGET_H
#define SPILLCONTROLWIDGET_H

#include <QWidget>

#include "device-modules/SpillControl/SpillConfig.h"
#include "fsm/FsmState.h"
#include <optional>

namespace Ui {
class SpillControlWidget;
}

struct SpillStatus;

class SpillControlWidget : public QWidget
{
    Q_OBJECT

public:
    explicit SpillControlWidget(QWidget *parent = nullptr);
    virtual ~SpillControlWidget() override;

public slots:
    void updateRemoteBlocking(bool remoteConnected) const;
    void setSpillCfg(const SpillConfig &newCfg);
    void onFsmStateChange(const FsmState &state);
    void setSpillStatus(const std::optional<SpillStatus> &st);
    void setShowAllPhases(bool show);

private slots:
    // Spill
    void on_spinBoxSpillDurPhase1_valueChanged(int arg1);
    void on_spinBoxSpillDurPhase2_valueChanged(int arg1);
    void on_spinBoxSpillDurPhase3_valueChanged(int arg1);
    void on_spinBoxSpillDurPhase4_valueChanged(int arg1);
    void on_spinBoxSpillDurPhase5_valueChanged(int arg1);
    // Soft Spill
    void on_groupBoxSoftSpill_toggled(bool arg1);
    void on_spinBoxPeriodTimer_valueChanged(int arg1);

    void on_radioButtonSpillSourceAuto_toggled(bool checked);
    void on_radioButtonSpillSourceTtl_toggled(bool checked);
    void on_radioButtonSpillSourceWr_toggled(bool checked);
    void on_radioButtonSpillSourceTimer_toggled(bool checked);
    void on_radioButtonSpillSourceGate_toggled(bool checked);
    void on_radioButtonSpillSourceInfinite_toggled(bool checked);

    void setSpillMode(const SpillConfig::spill_sel_t &sel);

signals:
    void spillCfgChanged(const SpillConfig &config);

private:
    void restoreGuiFromConfig();
    void displaySpillSource(const std::optional<SpillStatus> &st);
    void updatePhasesVisibility() const;

    Ui::SpillControlWidget *ui;
    SpillConfig cfg;
    int blockSignal=0;
    bool showAllPhases = true;
};

#endif // SPILLCONTROLWIDGET_H
