/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef FLASHIDDATA_H
#define FLASHIDDATA_H

#include <cstdint>
#include <string>

struct FlashIdData
{
    static constexpr const uint32_t FLASH_SECTOR_SIZE = 0x10000; // 64 kByte
    static constexpr const uint32_t FLASH_BLOCK_SIZE = 256;      // bytes
    static_assert(FLASH_SECTOR_SIZE % FLASH_BLOCK_SIZE == 0, "Sector size is not integer number of blocks");

    std::string getMemoryType() const
    {
        switch (memoryType) {
        case 0xBA:
            return "3V";
        case 0xBB:
            return "1.8V";
        default:
            return "unknown";
        }
    }
    uint32_t getBlockSize() const
    {
        return FLASH_BLOCK_SIZE;
    }
    uint32_t getSectorSize() const
    {
        return FLASH_SECTOR_SIZE;
    }
    uint32_t getFlashSizeBytes() const
    {
        return getFlashCapacity_Mib() * 131072;
    }

    uint32_t getFlashCapacity_Mib() const
    {
        if (isValid()) {
            if (memoryCapacity >= 0x17 && memoryCapacity <= 0x19)
                return 1 << (memoryCapacity - 0x11);
            if (memoryCapacity >= 0x20 && memoryCapacity <= 0x22)
                return 1 << (memoryCapacity - 0x17);
        }
        return 0;
    } // in Mb

    bool isValid() const
    {
        return manufacturerID == 0x20 && remaingDataSize == 0x10 && deviceConfigInfo == 0x0;
    }

    uint8_t manufacturerID = 0;
    uint8_t memoryType = 0;
    uint8_t memoryCapacity = 0;
    uint8_t remaingDataSize = 0;
    uint8_t extendedId = 0;
    uint8_t deviceConfigInfo = 0;
    uint8_t customizedFactoryData[14] = {0};
};

#endif // FLASHIDDATA_H
