#include "DeviceStatusZmqPublisher.h"

#include <QTimer>
#include <QJsonDocument>
#include <QJsonValue>

#include "base/AbstractConfigConverter.h"

DeviceStatusZmqPublisher::DeviceStatusZmqPublisher(QObject *parent)
    : ZmqPublisher(parent)
{
    setObjectName("ZmqDevStatPub");
    QTimer::singleShot(0, this, &DeviceStatusZmqPublisher::publish_event);
}

void DeviceStatusZmqPublisher::setStatus(const ModularDeviceStatus &r)
{
    pendingStatus[r.getDeviceIndex()] = r;
}

void DeviceStatusZmqPublisher::publish_event()
{
    sentDevStatus();
    pendingStatus.clear();
    QTimer::singleShot(1000, this, &DeviceStatusZmqPublisher::publish_event);
}

void DeviceStatusZmqPublisher::sentDevStatus()
{
    if(!socket) return;
//    if(!cfg.send_all) {
//        if(zmqSem->available() <= 1)
//            return;
//    }

    for(auto &st : pendingStatus) {
        QVariantMap devStatus{{"status",QVariantMap{{"runTime", getDevStatus(st)}}}};
        auto jsonObj = AbstractConfigConverter<int>::fromVariantMap(devStatus);

        QString mess_str = QString("status.%1 %2")
                .arg(st.getDeviceIndex().toString())
                .arg(QString(QJsonDocument(jsonObj).toJson(QJsonDocument::Compact)));

        publish(mess_str);
    }
}

QVariantMap DeviceStatusZmqPublisher::getDevStatus(const ModularDeviceStatus &devSt)
{
    QVariantMap devRunStatus;
    {
        const auto &temps = devSt.getTemps();
        QVariantMap tempStatus;
        for(auto it=temps.cbegin(); it!=temps.cend(); ++it) {
            auto id = it.key();
            tempStatus[id] = it.value();
        }
        devRunStatus["temp"] = tempStatus;
    }
    {
        QVariantMap timeStatus;
        timeStatus["app"] = QDateTime::currentDateTime().toTime_t();

        // WrStatus SavedTime in TRC module:
        // 0-CurTime, 1-RunStart, 2-SpillStart, 3-SpillEnd
        if (devSt.wrStatus.has_value()) {
            auto curDevTime = devSt.wrStatus->savedTimeCode(0);
            if(curDevTime.isNull())
                curDevTime = devSt.wrStatus->timeCode(); // timeCode - doesn't locked on readout
            timeStatus["WR"] = QVariant::fromValue(curDevTime)
                                   .value<QJsonObject>();
        }
        devRunStatus["time"] = timeStatus;
    }

    return devRunStatus;
}
