//
//    Copyright 2015 AFI Electronics
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef ABSTRACTRAWREADER_H
#define ABSTRACTRAWREADER_H

#include <QObject>
#include <QMap>
#include <QMetaType>

#include "TextTableStats.h"

class QSemaphore;

class AbstractRawReader : public QObject
{
    Q_OBJECT
public:
    explicit AbstractRawReader(QSemaphore * freeSpace, QObject *parent = 0);
    ~AbstractRawReader();

protected slots:
    void dataAcquired(const QByteArray &d);
    void analyzeData();
    void listenStateUpdated(bool listen);
    virtual void reset();
    virtual void clearStat();
    virtual void restartEventPrint() {}

signals:
    void textTableStatsUpdated(const TextTableStats &);

protected:
    QString rawDataErrorString(qint64 pos, const QString &text);
    virtual int readEvent(int startPos) = 0; /// return offset to next unread data
    virtual int findSync (int startPos) = 0; /// return end position of search

    QByteArray data;

    qint64 streamPos; /// absolute position in stream
//    const quint32 *curWordPtr;
//    qint64 curPos; // cur Word pos in data block
    bool syncLost;
    TextTableStats stats;

private slots:
    void sendStats();

private:
    void releaseReadData(int offset);

    QSemaphore * const freeSpaceSem;
    bool listenEnabled;
    TextTableStats sentStats;
};

#endif // ABSTRACTRAWREADER_H
