#ifndef FILEREADER_H
#define FILEREADER_H

#include <QFile>
#include <QVector>
#include <QMetaType>

class QSemaphore;
class QTimer;

struct FileReaderRecord
{
    FileReaderRecord(qint64 _pos=0, qint64 _len=0, quint32 _sync=0, quint32 _evNum=0):
        pos(_pos), length(_len), sync(_sync), evNum(_evNum)
    {}
    qint64 pos;
    qint64 length;
    quint32 sync;
    quint32 evNum; // 0 in case of non regular event block
};

class FileReader : public QObject
{
    Q_OBJECT
public:
    FileReader(QSemaphore *sem);

public slots:
    void setFileName(QString fileName);
    void readRecord(int record, int count=1);
    void setPackSizeLimit(qint64 limit);
    void closeAll();
    void closeFile();

private slots:
    void readFile();

signals:
    void dataAccuired(QByteArray);
    void eventRead(quint32 ev_first, quint32 ev_last, qint64 size);
    void fileMapUpdated(qint64 fileSize, QString lastModify, int records, double decodedPart);

private:
    void updateFileMap();

    QSemaphore * const freeRawDataSem;
    QTimer *readFileTimer;
    QFile dataFile;
    std::vector<FileReaderRecord> records;
    bool doCloseAll;
    qint64 pack_size_limit;
};

#endif // FILEREADER_H
