#ifndef ANALYZERSTAT_H
#define ANALYZERSTAT_H

#include <QElapsedTimer>
#include <QMap>

#include "base/DeviceIndex.h"

struct AnalyzerChStat
{
    void clear()
    {
        adc_hits = 0;
        tdc_hits = 0;
        msc_hits = 0;
    }
    quint32 adc_hits = 0;
    quint32 tdc_hits = 0;
    quint32 msc_hits = 0;
};

struct AnalyzerDevStat
{
    void clear()
    {
        dev_ch_hits.clear();
        err_cnt.clear();
    }
    QMap<int, AnalyzerChStat> dev_ch_hits; // per channel
    QMap<quint32, QList<QPair<quint32, quint32>>>
        err_cnt; //stores evNum ranges per error word (<0x<HptdId><HptdcErr>, {<100,150>, {200,250}, ...}>)
};

struct AnalyzerStat
{
    void clear()
    {
        eventDecoded = 0;
        sizeDecoded = 0;
        decodeTimeNs = 0;
        decodeLoop = 0;
        workerLoop = 0;
        formatErrorCnt = 0;
        dev_stat.clear();
        ttvxsList.clear();

        timer.invalidate();
        skip = 0;
        pass = 0;
    }

    qint64 eventDecoded = 0;
    qint64 sizeDecoded = 0;
    quint32 lastEvNum = 0;
    qint64 decodeTimeNs = 0; // decode time 200-300 us per max adcWin
    qint64 workerLoop = 0;
    qint64 decodeLoop = 0;
    int formatErrorCnt = 0;
    QMap<DeviceIndex, AnalyzerDevStat> dev_stat;
    QList<DeviceIndex> ttvxsList;

    QElapsedTimer timer;
    qint64 skip = 0;
    qint64 pass = 0;
};
Q_DECLARE_METATYPE(AnalyzerStat)
#endif // ANALYZERSTAT_H
