#ifndef DEVICECALIBRATION_H
#define DEVICECALIBRATION_H

#include <QMap>
#include <QMetaType>
#include <QVector>

#include "base/DeviceIndex.h"

struct TempStatistic
{
    TempStatistic() {
        temps.resize(LIMIT);
    }
    void addTemp(double t) {
        temps[cnt] = t;
        if(++cnt>=LIMIT)
            update_mean();
    }
    void update_mean() {
        mean = 0;
        mean = std::accumulate(temps.begin(), temps.end(), mean);
        mean /= LIMIT;
        cnt=0;
        initialized=true;
    }
    bool temp_valid(double t) const {
        return std::abs(mean-t)<1;
    }

    const static int LIMIT=1000;
    QVector<double> temps;
    double mean=0;
    bool initialized=false;
    int cnt=0;
};

class DeviceCalibration
{
public:
    DeviceCalibration(DeviceIndex i=DeviceIndex()):
        index(i)
    {
        nch = 0;
//        nch = 16; from tqdc
        clearBase();
    }

//from tqdc
    void clearBase(){
        x1.clear();
        x4.clear();
        thrOffset.clear();
        for(int ch=0; ch<nch; ++ch) {
            x1[ch] = 0;
            x4[ch] = 0;
            thrOffset[ch] = 1920;
        }
    }
    bool hasValidChInlCal(int ch) const {
        auto it = inl.constFind(ch);
        if(it == inl.cend())
            return false;
        return isValidChInlCal(*it);
    }
    static bool isValidChInlCal(const QVector<double> &vals) {
        for(auto v : vals)
            if(!qFuzzyIsNull(v))
                return vals.size() == 1024;
        return false;
    }

    DeviceIndex index;
    int nch;
    QMap<int, QString> inlSource; // by ch
    QMap<int, QVector<double> > inl; // by ch
    QMap<int, uint > hitCnt;         // by ch
    QMap<uint, quint16> errMask; // by hptdc_id (used only for INL saving)

    QMap<int, double> minTemp; // by tempId; 0-oneWireID; 1,2,3...-mcu
    QMap<int, double> maxTemp;
    QMap<int, TempStatistic> tempStat; // to fix SPI statistic erros for temp values
    QSet<uint> fw_versions;
    QSet<uint> bmc_versions;

//from tqdc
    QMap<int, double> x1; // ADC baseline for gain x1
    QMap<int, double> x4; // ADC baseline for gain x4
    QMap<int, int> thrOffset; // thrOffset // in dacValue 0-4095
};
Q_DECLARE_METATYPE(DeviceCalibration)

typedef QMap<DeviceIndex, DeviceCalibration > InlResults;
Q_DECLARE_METATYPE(InlResults)

typedef QMap<DeviceIndex, DeviceIndex> TdcTtvxsMapType;
Q_DECLARE_METATYPE(TdcTtvxsMapType)

#endif // DEVICECALIBRATION_H
