/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef ABSTRACTSIGNALROUTER_H
#define ABSTRACTSIGNALROUTER_H

#include "base/DeviceIndex.h"

class ConfigurationManager;
class DeviceDispatcher;
class LldpDiscovery;
class RemoteControlServer;

class AbstractSignalRouter
{
public:
    virtual ~AbstractSignalRouter() = default;

    void setConfigurationManager(ConfigurationManager *p)
    { configurationManager = p; }
    void setDeviceDispatcher(DeviceDispatcher *p)
    { deviceDispatcher = p;}
    void setRemoteControlServer(RemoteControlServer *p)
    { remoteControlServer = p; }

    virtual void connectAll();
    virtual void connect_ConfigurationDispatcher() = 0;
    virtual void connect_FSM() = 0;
    virtual void connect_PnpServer() = 0;
    virtual void connect_RemoteControlServer() = 0;
    virtual void connect_deviceDispatcher();

protected:
    virtual DeviceIdSet supportedDeviceIds() const { return {}; }
    LldpDiscovery &lldpDiscovery();

    ConfigurationManager *configurationManager = nullptr;
    DeviceDispatcher *deviceDispatcher = nullptr;
    RemoteControlServer *remoteControlServer = nullptr;
};

#endif // ABSTRACTSIGNALROUTER_H
