#include "BaseSignalRouter.h"

#include <QDialog>

#include "configuration-manager/ConfigurationManager.h"
#include "fsm/BaseFsm.h"
#include "remote-control-server/RemoteControlServer.h"
#include "lib-common/BaseAppCore.h"
#include "lib-common/PnpBaseMainWindow.h"
#include "pnp-server/PnpDiscovery.h"

void BaseSignalRouter::setBaseAppCore(BaseAppCore *p)
{
    baseAppCore = p;
    setRemoteControlServer(baseAppCore->remoteControlServer);
}

void BaseSignalRouter::setPnpBaseMainWindow(PnpBaseMainWindow *p)
{
    pnpBaseMainWindow = p;
    setConfigurationManager(pnpBaseMainWindow->configuration_manager);
}

void BaseSignalRouter::connectAll() const
{
    connect_FSM();
    connect_MainWindow();
    connect_PnpServer();
    connect_RemoteControlServer();

//    // File menu
//    if (readoutAppMainWindow && configurationManager) {
//        connect(readoutAppMainWindow, &ReadoutAppMainWindow::actionConfigurationManager,
//                configurationManager, &QDialog::show);
//    }

    // Tools menu
//    if (readoutAppMainWindow && mStreamStatisticDialog) {
//        connect(readoutAppMainWindow, &ReadoutAppMainWindow::actionMstreamStatistics,
//                mStreamStatisticDialog, &QDialog::show);
//    }

//    if (readoutAppCore) {
//        connect(readoutAppCore, &ReadoutAppCore::runStateChanged,
//                mStreamStatisticDialog, &MStreamStatisticDialog::set_run_state);
//    }

//    if (readoutAppMainWindow) {
//        connect(readoutAppMainWindow, &ReadoutAppMainWindow::set_run_state,
//                mStreamStatisticDialog, &MStreamStatisticDialog::set_run_state); // TODO: core
//    }
}

void BaseSignalRouter::connect_FSM() const
{
    if (!mainFsm)
        return;
    if(baseAppCore) {
        connect(mainFsm, &BaseFsm::stateChanged,
                baseAppCore, &BaseAppCore::onFsmStateChange);
    }
    if (remoteControlServer) {
        connect(mainFsm, &BaseFsm::stateChanged,
                remoteControlServer, &RemoteControlServer::onFsmStateChange);
    }
}

void BaseSignalRouter::connect_MainWindow() const
{
    if (pnpBaseMainWindow && configurationManager) {
        connect(pnpBaseMainWindow, &PnpBaseMainWindow::actionConfigurationManager,
                configurationManager, &QDialog::show);
    }
}

void BaseSignalRouter::connect_PnpServer() const
{
    if (baseAppCore)
        PnpDiscovery::subscribeMyProgram(baseAppCore);
}

void BaseSignalRouter::connect_RemoteControlServer() const
{
    if (!remoteControlServer)
        return;

    if (baseAppCore) {
        connect(remoteControlServer, &RemoteControlServer::connectionStatusUpdated,
                baseAppCore, &BaseAppCore::remoteStatusUpdated);
        connect(baseAppCore, &BaseAppCore::allowRemoteControl,
                remoteControlServer, &RemoteControlServer::setAllowRemoteControl);
        connect(baseAppCore, &BaseAppCore::sendRunDocumentToRc,
                remoteControlServer, &RemoteControlServer::config_transfer);
        connect(baseAppCore, &BaseAppCore::rcProgramStateChanged,
                remoteControlServer, &RemoteControlServer::onRcProgramStateChange);
    }
}

void BaseSignalRouter::removeRemoteControlServer()
{
    Q_ASSERT(remoteControlServer);
    // TODO: use disconnect(remoteControlServer, nullptr, nullptr, nullptr);
    disconnect(remoteControlServer, &RemoteControlServer::startRequest);
    disconnect(remoteControlServer, &RemoteControlServer::stopRequest);
    disconnect(remoteControlServer, &RemoteControlServer::connectionStatusUpdated);
    remoteControlServer = nullptr;
}
