/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef DEVICESTATUSTABLEVIEW_H
#define DEVICESTATUSTABLEVIEW_H

#include <QTableView>
#include "base/DeviceIndexEnabled.h"

class DeviceIndex;
class DeviceIndexTableModel;
class ModularDeviceStatus;
class MStreamStat;
class MergeColumnsTableModel;
class QSortFilterProxyModel;
class ReceiverStat;

class DeviceStatusTableView : public QTableView
{
    Q_OBJECT
public:
    explicit DeviceStatusTableView(QWidget *parent = nullptr);
    QSize sizeHint() const override;

signals:
    void setDeviceList(const DeviceIndexEnabledMap &new_devices);
    void deviceListChanged(const DeviceIndexEnabledMap &new_devices);
    void currentDeviceIndexChanged(const DeviceIndex &deviceIndex);
    void setStatus(const ModularDeviceStatus &baseStatus);
    void setMultiStatus(const QMap<DeviceIndex, ModularDeviceStatus> &statusMap);
    void stat_updated(const MStreamStat &ms_stat);
    void stat_from_receiver_updated(const ReceiverStat &stat);
    void mstreamConnectionStateChanged(const DeviceIndex &deviceIndex, bool);

public slots:
    void hideColumn(int column);
    void showColumn(int column);

protected:
    void currentChanged(const QModelIndex &current, const QModelIndex &previous) override;
    DeviceIndex deviceIndexOf(const QModelIndex &index) const;
    void addSourceModel(DeviceIndexTableModel *model);
    void hideModelColumn(QAbstractItemModel *model, int column);
    void showModelColumn(QAbstractItemModel *model, int column);
    void saveColumnState(int column, bool shown = true);
    QString columnSettingsKey(int column);
    QVariant loadColumnState(int column);
    void resetColumnState();
    QMap<int, bool> defaultColumnState() const;

    MergeColumnsTableModel * const mergeModel;
    QSortFilterProxyModel * const proxyModel;

protected slots:
    virtual void deferredInit();

private slots:
    void updateSizeHint();

private:
    void setDefaultColumnState();
    DeviceIndexTableModel *createEnableModel();
    DeviceIndexTableModel *createDiscoverModel();
    DeviceIndexTableModel *createStatusModel();
    DeviceIndexTableModel *createMStreamModel();

    QList<DeviceIndexTableModel *> subModels;
};

#endif // DEVICESTATUSTABLEVIEW_H
