#ifndef RCCLIENTTREEMODEL_H
#define RCCLIENTTREEMODEL_H

#include <QAbstractItemModel>

#include "remote-control-server/RcProgramState.h"

struct RcProgramStateTreeItem;
class RcClientPath;

class RcClientTreeModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    enum Columns{
        COL_NAME, COL_FSM,
        COL_IS_GLOB_TRIG,
        COL_TOTAL
    };

    explicit RcClientTreeModel(QObject *parent = nullptr);
    ~RcClientTreeModel();

    virtual QModelIndex index(int row, int column,
                                          const QModelIndex &parent = QModelIndex()) const;
    virtual QModelIndex parent(const QModelIndex &child) const;

    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const;
    virtual int columnCount(const QModelIndex &parent = QModelIndex()) const;

    virtual QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
    virtual bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole);

    Qt::ItemFlags flags(const QModelIndex &index) const override;
    virtual QVariant headerData(int section, Qt::Orientation orientation,
                                int role = Qt::DisplayRole) const override;

    void setEnabled(bool en);

public slots:
    void clientRcProgramStateUpdated(const RcProgramState &state);
    void setMyClientIndex(const ClientIndex &index);

signals:
    void globTrigsChanged(const ClientIndexSet &) const;

private:
    const RcProgramState *getStateByPath(const RcClientPath &p) const;
    QVariant data_DisplayRole(const RcProgramStateTreeItem *item, int column) const;
    QVariant data_CheckStateRole(const RcProgramStateTreeItem *item, int column) const;
    QVariant data_ForegroundRole(const RcProgramStateTreeItem *item, int column) const;
    QVariant data_FontRole(const RcProgramStateTreeItem *item, int column) const;
    QVariant itemColor(const RcProgramStateTreeItem *item) const;
    void sendDataChanged(const QModelIndex &index = {});
    QSet<QModelIndex> clientIndexToModelIndex(ClientIndexSet *cis, const QModelIndex &i = {}) const;

    ClientIndex myClientIndex;
    ClientIndexSet globTrigs;
    RcProgramState rcProgramState;
    bool enable = true;
    RcProgramStateTreeItem * mRootItem;
};

#endif // RCCLIENTTREEMODEL_H
