#ifndef MLINK_FRAME_HEADER_H
#define MLINK_FRAME_HEADER_H

#include <cstdint>

// M-Link frame header
typedef struct {
    uint16_t type; // Frame type. Transferred second.
    uint16_t sync; // Frame sync. Must be set equal to ML_FRAME_SYNC. Transferred first on 16-bit bus.
    uint16_t seq; // Frame sequence number. Incremented on every next frame sent in one direction.
    uint16_t len; // Frame length in 32-bit words. Valid length is 4 – 1023 (data size 0-1019)
    uint16_t src; // Source address
    uint16_t dst; // Destination address
} frame_hdr_t;

constexpr const uint16_t ML_FRAME_SYNC = 0x2A50;
constexpr const int ML_FRAME_HEADER_WORDS = 3;
constexpr const int ML_FRAME_TRAILER_WORDS = 1;

constexpr const int ML_MIN_RAW_FRAME_WORDS = (ML_FRAME_HEADER_WORDS+ML_FRAME_TRAILER_WORDS);
constexpr const int ML_MIN_RAW_FRAME_OCTETS = (ML_MIN_RAW_FRAME_WORDS*4);

// Maximum number of words in frame including header, crc. See M-Link specification.
constexpr const int ML_MAX_RAW_FRAME_WORDS = 350;
constexpr const int ML_MAX_RAW_FRAME_OCTETS = (ML_MAX_RAW_FRAME_WORDS*4);

// Maximum number of payload data words.
constexpr const int ML_MAX_FRAME_DATA_WORDS = (ML_MAX_RAW_FRAME_WORDS-(ML_FRAME_HEADER_WORDS+ML_FRAME_TRAILER_WORDS));

#endif // MLINK_FRAME_HEADER_H
