/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef MLINKSTATEFSM_H
#define MLINKSTATEFSM_H

#include <QObject>
#include <QElapsedTimer>

class MlinkStateFsm : public QObject
{
    Q_OBJECT
public:
    enum State {Closed, Detected, Connected, Failed};

    explicit MlinkStateFsm(QObject *parent = nullptr)
        : QObject(parent)
    {
        stateTimer.start();
    }

    bool changeState(State newState);
    State state() const {return curState; }
    static QString stateStr(State s);
    qint64 stateTimeMs() const {return stateTimer.elapsed(); }

signals:
    void stateChanged(MlinkStateFsm::State oldState, MlinkStateFsm::State newState);

public slots:

private:
    State curState = Closed;
    QElapsedTimer stateTimer;
};

QDebug operator<<(QDebug dbg, MlinkStateFsm::State state);

#endif // MLINKSTATEFSM_H
