//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//


#include "regiopacket.h"

#include <iostream>
#include <iomanip>
#include <sstream>
#include <algorithm>
#include <iterator>

#include <QDebug>
#include <QTextStream>

std::ostream &operator<< (std::ostream & os, const mlink::PacketRawData &p)
{
    std::ostringstream sstr;
    sstr << "PacketRawData[" << p.size() << "] = {" << std::endl;
    sstr << std::hex;
    //    std::copy(p.begin(), p.end(), std::ostream_iterator<PacketRawDataType>(sstr, " "));
    size_t n = 0;
    for (auto it : p) {
        if (n%8==0) sstr << std::endl << "  ";
        sstr << std::setw(8) << std::setfill('0') << it << " ";
        n++;
    }
    if (n%8!=0) sstr << std::endl;
    sstr << "};";
    os << sstr.str();
    return os;
}

std::ostream &operator<< (std::ostream & os, const mlink::RegIoPacket &p)
{
    std::ostringstream sstr;
    sstr << std::hex;
    sstr << "type " << std::setw(4) << std::setfill('0') << p.type
            << ", seq " << std::setw(4) << std::setfill('0') << p.seq << ", ";
    if (p.src < p.dst) {
        sstr << std::setw(4) << std::setfill('0') << p.src
                << "->" << std::setw(4) << std::setfill('0') << p.dst;
    } else {
        sstr << std::setw(4) << std::setfill('0') << p.dst
                << "<-" << std::setw(4) << std::setfill('0') << p.src;
    }
    sstr << ", " << std::vector<mlink::PacketRawDataType>(p.data.begin(), p.data.end());
    os << sstr.str();
    return os;
}

QTextStream &operator<< (QTextStream & os, const mlink::RegIoPacket &p)
{
    std::ostringstream sstr;
    sstr << p;
    os << QString::fromStdString(sstr.str());
    return os;
}

QDebug operator<< (QDebug os, const mlink::RegIoPacket &p)
{
    std::ostringstream sstr;
    sstr << p;
    os << QString::fromStdString(sstr.str());
    return os;
}

void mlink::RegIoPacket::dump(const std::string &description) const
{
    std::string s;
    if (! description.empty())
        s = " (" + description + ")";
    std::cerr << "--- Packet dump begin" << s << " ---" << std::endl;
    std::cerr << (*this);
    std::cerr << "--- Packet dump end  " << s << " ---" << std::endl;
}
