/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef ABSTRACTCONFIGDB_H
#define ABSTRACTCONFIGDB_H

#include <QObject>

#include "BaseConfig.h"
#include "base/ConfigSelector.h"
#include "base/ConfigDocument.h"

class QTimer;
struct RunDocument;

// FIXME: change BaseConfig inheritance to composition
class AbstractConfigDb : public QObject, protected BaseConfig
{
    Q_OBJECT
public:
    explicit AbstractConfigDb(QObject *parent = nullptr);
    virtual ~AbstractConfigDb() override ;
    void write_config(QJsonObject obj_);

    ConfigSelector getConfigSelector() const {
        return cs;
    }
    DaqModule getType() const {
        return cs.getType();
    }
    ProgramIndex getIndex() const {
        return getConfigIndexName().getIndex();
    }
    ConfigurationName getConfigurationName() const {
        return getConfigIndexName().getConfigurationName();
    }
    ConfigIndexName getConfigIndexName() const {
        return cs.getConfigIndexName();
    }
    ClientIndex getClientIndex() const {
        return getConfigSelector().getClientIndex();
    }
    QString getWindowTitle() const {
        return cs.toWindowTitle();
    }

public slots:
//    void setConfig(const T &newConfig) {}
    void configSelectorChanged(const ConfigIndexName &cin);
    virtual void update_all();
    void saveRunDocument(const RunDocument &rd);

private slots:
    void startUpInit();
    void write_request();

signals:
//    void configChanged(const T &);
    void switch_to_config(const ConfigIndexName &conf) const;

protected:
    virtual void update_config_from_db() = 0;

//    T config;
    ConfigSelector cs;
    bool initialized = false;
    QTimer * const write_timer;
    bool write_request_flag;
    QJsonObject obj;
};

#endif // ABSTRACTCONFIGDB_H
