/*
**    Copyright 2011-2022 Ivan Filippov
**    Copyright 2011-2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef PACKETBUFFER_H
#define PACKETBUFFER_H

#include "types.h"

struct PacketBuffer
{
    void setMStreamV2Packet()
    {
        if (len <= (sizeof(MLinkFrameHdr)+4)) { payload = nullptr; payloadSize = 0; return; }
        auto mh = reinterpret_cast<MLinkFrameHdr *>(buf);
        if (mh->sync != ML_FRAME_SYNC || mh->type != FRAME_TYPE_MSTREAM || mh->len < 4)
            { payload = nullptr; payloadSize = 0; return; }
        payload = &buf[sizeof(MLinkFrameHdr)];
        payloadSize = len - sizeof(MLinkFrameHdr) - 4; // cut 4 bytes of CRC at end
    }
    void setMStreamV3Packet() {
        payload = &buf[0];
        payloadSize = len;
    }
    void clear() {
        len = 0;
        payloadSize = 0;
    }
    bool dataValid() const {
        return payload != nullptr && payloadSize >= sizeof(MStreamFragmentHeader);
    }
    MStreamFragmentHeader *first() const {
        if (!dataValid()) return nullptr;
        return (MStreamFragmentHeader*)payload;
    }
    char *data(const MStreamFragmentHeader *h) const {
        if (!buf || !h) return nullptr;
        const auto headerOffset = ((char *)h - buf);
        const auto dataOffset = headerOffset + sizeof(MStreamFragmentHeader);
        if (dataOffset + h->fragmentLength > len) return nullptr;
        return &buf[dataOffset];
    }
    MStreamFragmentHeader *next(const MStreamFragmentHeader *h) const {
        if (!buf || !h) return nullptr;
        const auto headerOffset = ((char *)h - buf);
        const auto dataOffset = headerOffset + sizeof(MStreamFragmentHeader);
        if (dataOffset > len)
            return nullptr;
        // safe to read header now
        auto nextOffset = dataOffset + h->fragmentLength;
        if (nextOffset + sizeof(MStreamFragmentHeader) > len)
            return nullptr; // expected
        return (MStreamFragmentHeader*)&buf[nextOffset];
    }

    char *buf = nullptr;
    uint len = 0;
    char *payload = nullptr;
    uint payloadSize = 0;
};
Q_DECLARE_METATYPE(PacketBuffer)

#endif // PACKETBUFFER_H
