#ifndef PNPDISCOVERY_H
#define PNPDISCOVERY_H

#include <QObject>

#include "PNPMessage.h"
#include "PnpDiscoveryFilter.h"

class PnpDiscoveryPrivate;

class PnpDiscovery : public QObject
{
    Q_OBJECT
public:
    // Singleton
    static PnpDiscovery & instance()
    {
        // thread-safe since C++11
        static PnpDiscovery obj;
        obj.setObjectName("PnpDiscovery");
        return obj;
    }

    // delete copy and move constructors and assign operators
    PnpDiscovery(PnpDiscovery const &) = delete;
    PnpDiscovery(PnpDiscovery &&) = delete;
    PnpDiscovery &operator=(PnpDiscovery const &) = delete;
    PnpDiscovery &operator=(PnpDiscovery &&) = delete;


    // subscribe with permanent filter (post-processing after variable filter)
    template<class T>
    static void subscribeMyProgram(T *obj, const DaqModuleSet &id = {}) {
        auto &ref = instance();

        connect(obj, &T::updateProgramDescription,
                &ref, &PnpDiscovery::updateMyProgramDescription);
        connect(obj, &T::closeProgram,
                &ref, &PnpDiscovery::closeMyProgram,
                Qt::UniqueConnection);
    }

    template<class T>
    static void subscribeCacheUpdaed(T *obj, const DaqModuleSet &id = {}) {
        auto &ref = instance();

        if (id.isEmpty()) {
            connect(&ref, &PnpDiscovery::cacheUpdated,
                    obj, &T::pnpCacheUpdated,
                    Qt::UniqueConnection);
        } else {
            auto filter = new PnpDiscoveryFilter(id, &ref);

            connect(&ref, &PnpDiscovery::cacheUpdated,
                    filter, &PnpDiscoveryFilter::pnpCacheUpdated,
                    Qt::UniqueConnection);

            connect(filter, &PnpDiscoveryFilter::pnpCacheUpdatedFiltered,
                    obj, &T::pnpCacheUpdated,
                    Qt::UniqueConnection);

            connect(obj, &T::destroyed,
                    filter, &QObject::deleteLater);
        }
    }

signals:
    void cacheUpdated(const PnpInfoCache &) const; // from PnpServer to obj
    void updateMyProgramDescription(const ProgramDescription &prog_decsr) const; // from obj to PnpServer
    void gotProgramClosed(const ProgramDescription &pd) const; // from PnpServer to obj
    void closeMyProgram(const ProgramDescription &pd) const; // from obj to PnpServer

protected:
    PnpDiscovery();
    virtual ~PnpDiscovery();

private:
    Q_DECLARE_PRIVATE(PnpDiscovery)
    PnpDiscoveryPrivate * const d_ptr;
};

#endif // PNPDISCOVERY_H
