/*
**    Copyright 2009-2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef WAVEFORMDATA_H
#define WAVEFORMDATA_H

#include <QString>
#include <QVector>
#include <string>
#include <vector>

class point_t
{
public:
    point_t() = default;
    point_t(const point_t &) = default;
    point_t(double x, double y) : x(x), y(y) {}
    double x = 0;
    double y = 0;
};

class WaveformData
{
public:
    WaveformData() = default;
    WaveformData(const WaveformData &) = default;
    WaveformData(const QString &dataName, int dataChannel = 0)
        : c(dataChannel), name(dataName)
    {}

    [[deprecated]] WaveformData(const std::string &dataName, int dataChannel = 0)
        : WaveformData(QString::fromStdString(dataName), dataChannel) {}
    WaveformData(const std::vector<double> &vy, const QString &dataName = {}, int dataChannel = 0);
    [[deprecated]] WaveformData(const std::vector<double> &vy, const std::string &dataName = {}, int dataChannel = 0)
        : WaveformData(vy, QString::fromStdString(dataName), dataChannel) {}
    WaveformData(const std::vector<double> &vx, const std::vector<double> &vy, const QString &dataName = {}, int dataChannel = 0);
    [[deprecated]] WaveformData(const std::vector<double> &vx, const std::vector<double> &vy, const std::string &dataName = {}, int dataChannel = 0)
        : WaveformData(vx, vy, QString::fromStdString(dataName), dataChannel) {}

    void simulate(unsigned int length = 20);
    void print() const;
    void setName(const QString &str) { name = str; }
    void setName(const std::string &str) { name = QString::fromStdString(str); }

    WaveformData& operator=(const WaveformData &) = default;

    unsigned int c; // channel
    std::vector<point_t> p; // points
    std::vector<double> t; // time marks
    QString name;
};

typedef std::vector<WaveformData> WaveformDataAll;

class WaveformDataSimulator
{
public:
    static WaveformDataAll run();
};

#endif // WAVEFORMDATA_H
